@@ -14,14 +14,12 @@ from time import sleep
 from sys import modules
 from os.path import dirname
 from logging import getLogger
-from os.path import join
-from datetime import datetime
 
 logger = getLogger(__name__)
 
 
 class BlokManagerException(LookupError):
-    """ Simple exception to BlokManager """
+    """Simple exception class for BlokManager """
 
     def __init__(self, *args, **kwargs):
         EnvironmentManager.set('current_blok', None)
@@ -29,7 +27,7 @@ class BlokManagerException(LookupError):
 
 
 class BlokManager:
-    """ Manage the bloks for one process
+    """Manage the bloks for one process
 
     A blok has a `setuptools` entrypoint, this entry point is defined
     by the ``entry_points`` attribute in the first load
@@ -46,11 +44,10 @@ class BlokManager:
     entry_points = None
     ordered_bloks = []
     auto_install = []
-    importers = {}
 
     @classmethod
     def list(cls):
-        """ Return the ordered bloks
+        """Return the ordered bloks
 
         :rtype: list of blok name ordered by loading
         """
@@ -58,18 +55,18 @@ class BlokManager:
 
     @classmethod
     def has(cls, blok):
-        """ Return True if the blok is loaded
+        """Return True if the blok is loaded
 
-        :param blok: blok name
+        :param blok: the name of the blok
         :rtype: bool
         """
         return blok and blok in cls.ordered_bloks or False
 
     @classmethod
     def get(cls, blok):
-        """ Return the loaded blok
+        """Return the loaded blok
 
-        :param blok: blok name
+        :param blok: the name of the blok
         :rtype: blok instance
         :exception: BlokManagerException
         """
@@ -80,9 +77,9 @@ class BlokManager:
 
     @classmethod
     def set(cls, blokname, blok):
-        """ Add a new blok
+        """Add a new blok
 
-        :param blokname: blok name
+        :param blokname: the name of the blok
         :param blok: blok instance
         :exception: BlokManagerException
         """
@@ -95,7 +92,7 @@ class BlokManager:
     @classmethod
     @log(logger, level='debug')
     def reload(cls):
-        """ Reload the entry points
+        """Reload the entry points
 
         Empty the ``bloks`` dict and use the ``entry_points`` attribute to
         load bloks
@@ -114,7 +111,7 @@ class BlokManager:
     @classmethod
     @log(logger, level='debug')
     def unload(cls):
-        """ Unload all the bloks but not the registry """
+        """Unload all the bloks but not the registry """
         cls.bloks = {}
         cls.ordered_bloks = []
         cls.entry_points = None
@@ -123,16 +120,20 @@ class BlokManager:
         RegistryManager.unload()
 
     @classmethod
-    def get_need_blok_linked_bloks(cls, blok):
+    def get_needed_blok_dependencies(cls, blok):
+        """Get all dependencies for the blok given
+
+        :param blok:
+        :return:
+        """
         for required in cls.bloks[blok].required:
-            if not cls.get_need_blok(required):
-                raise BlokManagerException(
-                    "Not %s required bloks found" % required)
+            if not cls.get_needed_blok(required):
+                cls.add_undefined_blok(required)
 
             cls.bloks[required].required_by.append(blok)
 
         for optional in cls.bloks[blok].optional:
-            if cls.get_need_blok(optional):
+            if cls.get_needed_blok(optional):
                 cls.bloks[optional].optional_by.append(blok)
 
         for conditional in cls.bloks[blok].conditional:
@@ -142,36 +143,59 @@ class BlokManager:
             cls.bloks[conflicting].conflicting_by.append(blok)
 
     @classmethod
-    def get_need_blok(cls, blok):
+    def blok_importers(cls, blok):
+        EnvironmentManager.set('current_blok', blok)
+
+        if not ImportManager.has(blok):
+            # Import only if the blok doesn't exists, do not reload here
+            mod = ImportManager.add(blok)
+            mod.imports()
+        else:
+            mod = ImportManager.get(blok)
+            mod.reload()
+
+    @classmethod
+    def get_needed_blok(cls, blok):
+        """Get and import/load the blok given with dependencies
+
+        :param blok:
+        :return:
+        """
         if cls.has(blok):
             return True
 
         if blok not in cls.bloks:
             return False
 
-        cls.get_need_blok_linked_bloks(blok)
+        cls.get_needed_blok_dependencies(blok)
         cls.ordered_bloks.append(blok)
-        EnvironmentManager.set('current_blok', blok)
-
-        if not ImportManager.has(blok):
-            # Import only if not exist don't reload here
-            mod = ImportManager.add(blok)
-            mod.imports()
-        else:
-            mod = ImportManager.get(blok)
-            mod.reload()
+        cls.blok_importers(blok)
 
         if cls.bloks[blok].autoinstall:
             cls.auto_install.append(blok)
 
         return True
 
+    @classmethod
+    def add_undefined_blok(cls, name):
+        blok = type(
+            'undefined-blok.%s' % name,
+            (UndefinedBlok,),
+            dict(name=name, required_by=[], optional_by=[], conditional_by=[],
+                 conflicting_by=[])
+        )
+        blok.__doc__ = "Blok undefined"
+        cls.set(name, blok)
+        # here they are not python module to load, but this action
+        # add the undefined blok in registryManager
+        cls.blok_importers(name)
+
     @classmethod
     @log(logger, level='debug')
     def load(cls, entry_points=('bloks',)):
-        """ Load all the bloks and import them
+        """Load all the bloks and import them
 
-        :param entry_points: Use by ``iter_entry_points`` to get the blok
+        :param entry_points: Used by ``iter_entry_points`` to get the blok
         :exception: BlokManagerException
         """
         if not entry_points:
@@ -210,14 +234,14 @@ class BlokManager:
         try:
             while bloks:
                 blok = bloks.pop(0)[1]
-                cls.get_need_blok(blok)
+                cls.get_needed_blok(blok)
 
         finally:
             EnvironmentManager.set('current_blok', None)
 
     @classmethod
     def getPath(cls, blok):
-        """ Return the path of the blok
+        """Return the path of the blok
 
         :param blok: blok name in ``ordered_bloks``
         :rtype: absolute path
@@ -225,42 +249,14 @@ class BlokManager:
         blok = cls.get(blok)
         return dirname(modules[blok.__module__].__file__)
 
-    @classmethod
-    def add_importer(cls, key, cls_name):
-        """ Add a new importer
-
-        :param key: key of the importer
-        :param cls_name: name of the model to import
-        """
-        cls.importers[key] = cls_name
-
-    @classmethod
-    def has_importer(cls, key):
-        """ Check if an importer  """
-        return True if key in cls.importers else False
-
-    @classmethod
-    def get_importer(cls, key):
-        """ Get the importer class name
-
-        :param key: key of the importer
-        :rtype: name of the model to import
-        :exception: BlokManagerException
-        """
-        if not cls.has_importer(key):
-            raise BlokManagerException(
-                "No importer found for the key %r" % key)
-
-        return cls.importers[key]
-
 
 class Blok:
-    """ Super class for all the bloks
+    """Super class for all the bloks
 
     define the default value for:
 
     * priority: order to load the blok
-    * required: list of the bloks needed to install this blok
+    * required: list of the bloks required to install this blok
     * optional: list of the bloks to be installed if present in the blok list
     * conditional: if all the bloks of this list are installed then install
       this blok
@@ -272,7 +268,7 @@ class Blok:
     optional = []
     conditional = []
     conflicting = []
-    name = None  # filled by the BlokManager
+    name = None  # set by the BlokManager
     author = ''
     logo = ''
 
@@ -281,73 +277,56 @@ class Blok:
 
     @classmethod
     def import_declaration_module(cls):
-        """ Do the python import for the Declaration of the model or other
+        """Do the python import for the Declaration of the model or other
         """
 
     def update(self, latest_version):
-        """ Call at the installation or update
+        """Called on install or update
 
-        :param latest_version: latest version installed, if the blok have not
-                               been installing the latest_version will be None
+        :param latest_version: latest version installed, if the blok has never
+                               been installed, latest_version is None
         """
 
     def pre_migration(self, latest_version):
-        """Call at update, before the automigration
+        """Called on update, before the auto migration
 
         .. warning::
 
             You can not use the ORM
 
-        :param latest_version: latest version installed, if the blok have not
-                               been installing the latest_version will be None
+        :param latest_version: latest version installed, if the blok has never
+                               been installed, latest_version is None
         """
 
     def post_migration(self, latest_version):
-        """Call at update, after the automigration
+        """Called on update, after the auto migration
 
-        :param latest_version: latest version installed, if the blok have not
-                               been installing the latest_version will be None
+        :param latest_version: latest version installed, if the blok has never
+                               been installed, latest_version is None
+        """
+
+    def update_demo(self, latest_version):
+        """Called on install or update to set or update demo data if
+        `System.Parameter.get("with-demo", False) is True`
+
+        :param latest_version: latest version installed, if the blok has never
+                               been installed, latest_version is None
+        """
+
+    def uninstall_demo(self):
+        """Called on uninstall demo data if
+        `System.Parameter.get("with-demo", False) is True`
         """
 
     def uninstall(self):
-        """ Call at the uninstallation
+        """Called on uninstall
         """
 
     def load(self):
-        """ Call at the launch of the application
+        """Called on start / launch
         """
 
-    def import_file(self, importer_name, model, *file_path, **kwargs):
-        """ Import data file
 
-        :param importer_name: Name of the importer (need installation of the
-                              Blok which have the importer)
-        :param model: Model of the data to import
-        :param \*file_path: relative path of the path in this Blok
-        :param \*\*kwargs: Option for the importer
-        :rtype: return dict of result
-        """
-        blok_path = BlokManager.getPath(self.name)
-        _file = join(blok_path, *file_path)
-        logger.info("import %r file: %r", importer_name, _file)
-        Importer = self.registry.get(BlokManager.get_importer(importer_name))
-        file_to_import = None
-        with open(_file, 'rb') as fp:
-            file_to_import = fp.read()
+class UndefinedBlok(Blok):
 
-        importer = Importer.insert(
-            model=model, file_to_import=file_to_import, **kwargs)
-        started_at = datetime.now()
-        res = importer.run(self.name)
-        stoped_at = datetime.now()
-        dt = stoped_at - started_at
-        logger.info("Create %d entries, Update %d entries (%d.%d sec)",
-                    len(res['created_entries']), len(res['updated_entries']),
-                    dt.seconds, dt.microseconds)
-        if 'error_found' in res and res['error_found']:
-            for error in res['error_found']:
-                logger.error(error)
-        else:
-            importer.delete()
-
-        return res
+    version = '0.0.0'
