@@ -2,6 +2,7 @@
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -52,7 +53,10 @@ class AnyBlokCore(Blok):
     logo = '../anyblok-logo_alpha_256.png'
 
     def pre_migration(self, latest_version):
-        if latest_version is not None and latest_version < '0.4.1':
+        if latest_version is None:
+            return
+
+        if latest_version < '0.4.1':
             self.pre_migration_0_4_1_fields_become_polymorphic(latest_version)
 
     def pre_migration_0_4_1_fields_become_polymorphic(self, latest_version):
@@ -102,7 +106,6 @@ class AnyBlokCore(Blok):
         from . import system  # noqa
         from . import authorization  # noqa
         from . import documentation  # noqa
-        from . import mixins  # noqa
 
     @classmethod
     def reload_declaration_module(cls, reload):
@@ -114,5 +117,3 @@ class AnyBlokCore(Blok):
         reload(authorization)
         from . import documentation
         reload(documentation)
-        from . import mixins
-        reload(mixins)
