@@ -1,6 +1,7 @@
 # This file is a part of the AnyBlok project
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2019 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -12,7 +13,7 @@ from anyblok.mapper import FakeColumn, FakeRelationShip
 from anyblok.relationship import RelationShip, Many2Many
 from anyblok.common import anyblok_column_prefix
 from ..exceptions import SqlBaseException
-from sqlalchemy.orm import aliased, ColumnProperty
+from sqlalchemy.orm import aliased, ColumnProperty, load_only
 from sqlalchemy.sql.expression import true
 from sqlalchemy import or_, and_, inspect
 from sqlalchemy_utils.models import NO_VALUE, NOT_LOADED_REPR
@@ -32,6 +33,8 @@ class uniquedict(dict):
 
 class SqlMixin:
 
+    __db_schema__ = None
+
     def __repr__(self):
         state = inspect(self)
         field_reprs = []
@@ -74,13 +77,21 @@ class SqlMixin:
     def define_table_args(cls):
         return ()
 
+    @classmethod
+    def define_table_kwargs(cls):
+        res = {}
+        if cls.__db_schema__ is not None:
+            res.update({'schema': cls.__db_schema__})
+
+        return res
+
     @classmethod
     def define_mapper_args(cls):
         return {}
 
     @classmethod
     def get_all_registry_names(cls):
-        models = [base.__registry_name__ for base in cls.__anyblok_bases__]
+        models = list(cls.__depends__)
         models.insert(0, cls.__registry_name__)
         return models
 
@@ -106,9 +117,12 @@ class SqlMixin:
                 res.append(f)
 
         if res:
-            return cls.registry.query(*res)
+            query = cls.registry.query(*res)
+        else:
+            query = cls.registry.query(cls)
 
-        return cls.registry.query(cls)
+        query.set_Model(cls)
+        return query
 
     is_sql = True
 
@@ -125,13 +139,15 @@ class SqlMixin:
 
         :rtype: SqlAlchemy aliased of the model
         """
-        return aliased(cls, *args, **kwargs)
+        alias = aliased(cls, *args, **kwargs)
+        alias.registry = alias._aliased_insp._target.registry
+        return alias
 
     @classmethod
     def get_where_clause_from_primary_keys(cls, **pks):
         """ return the where clause to find object from pks
 
-        :param \*\*pks: dict {primary_key: value, ...}
+        :param **pks: dict {primary_key: value, ...}
         :rtype: where clause
         :exception: SqlBaseException
         """
@@ -143,15 +159,29 @@ class SqlMixin:
 
         return [getattr(cls, k) == v for k, v in pks.items()]
 
+    @classmethod
+    def query_from_primary_keys(cls, **pks):
+        """return a Query object in order to get object from primary keys.
+
+        .. code::
+
+            query = Model.query_from_primary_keys(**pks)
+            obj = query.one()
+
+        :param **pks: dict {primary_key: value, ...}
+        :rtype: Query object
+        """
+        where_clause = cls.get_where_clause_from_primary_keys(**pks)
+        return cls.query().filter(*where_clause)
+
     @classmethod
     def from_primary_keys(cls, **pks):
         """ return the instance of the model from the primary keys
 
-        :param \*\*pks: dict {primary_key: value, ...}
+        :param **pks: dict {primary_key: value, ...}
         :rtype: instance of the model
         """
-        where_clause = cls.get_where_clause_from_primary_keys(**pks)
-        query = cls.query().filter(*where_clause)
+        query = cls.query_from_primary_keys(**pks)
         if query.count():
             return query.first()
 
@@ -161,7 +191,7 @@ class SqlMixin:
     def from_multi_primary_keys(cls, *pks):
         """ return the instances of the model from the primary keys
 
-        :param \*pks: list of dict [{primary_key: value, ...}]
+        :param *pks: list of dict [{primary_key: value, ...}]
         :rtype: instances of the model
         """
         where_clause = []
@@ -194,7 +224,7 @@ class SqlMixin:
         :type: list of the primary keys name
         """
         C = cls.registry.System.Column
-        query = C.query()
+        query = C.query().distinct(C.name).options(load_only(C.name))
         query = query.filter(C.model.in_(cls.get_all_registry_names()))
         query = query.filter(C.primary_key == true())
         return query.all().name
@@ -204,8 +234,8 @@ class SqlMixin:
         """ Return the information of the Field, Column, RelationShip """
         Field = cls.registry.System.Field
         res = {}
-        for base in cls.__anyblok_bases__:
-            query = Field.query().filter(Field.model == base.__registry_name__)
+        for registry_name in cls.__depends__:
+            query = Field.query().filter(Field.model == registry_name)
             res.update({x.name: x._description() for x in query.all()})
 
         query = Field.query().filter(Field.model == cls.__registry_name__)
@@ -386,6 +416,83 @@ class SqlMixin:
         query = query.limit(1)
         return query.one().ftype
 
+    @classmethod_cache()
+    def find_remote_attribute_to_expire(cls, *fields):
+        res = uniquedict()
+        _fields = []
+        _fields.extend(fields)
+        model = get_model_information(cls.registry, cls.__registry_name__)
+        while _fields:
+            field = _fields.pop()
+            field = field if isinstance(field, str) else field.name
+            _field = model[field]
+
+            if isinstance(_field, (Column, FakeColumn)):
+                _fields.extend(x for x, y in model.items()
+                               if (isinstance(y, RelationShip) and
+                                   not isinstance(y, Many2Many))
+                               for mapper in y.column_names
+                               if mapper.attribute_name == field)
+                if isinstance(_field, Column) and _field.foreign_key:
+                    rmodel = cls.registry.loaded_namespaces_first_step[
+                        _field.foreign_key.model_name]
+                    for rc in [x for x, y in rmodel.items()
+                               if isinstance(y, RelationShip)
+                               for mapper in y.remote_columns
+                               if mapper.attribute_name == field]:
+                        rfield = rmodel[rc]
+                        if isinstance(rfield, FakeRelationShip):
+                            res.add_in_res(rfield.mapper.attribute_name, [rc])
+                        elif (isinstance(rfield, RelationShip) and
+                              'backref' in rfield.kwargs):
+                            res.add_in_res(
+                                rfield.kwargs['backref'][0], [rc])
+
+            elif (isinstance(_field, RelationShip) and
+                  not isinstance(_field, Many2Many) and
+                  'backref' in _field.kwargs):
+                res.add_in_res(field, [_field.kwargs['backref'][0]])
+            elif isinstance(_field, FakeRelationShip):
+                res.add_in_res(field, [_field.mapper.attribute_name])
+
+        return res
+
+    @classmethod_cache()
+    def find_relationship(cls, *fields):
+        """ Find column and relation ship link with the column or relationship
+        passed in fields.
+
+        :param *fields: lists of the attribute name
+        :rtype: list of the attribute name of the attribute and relation ship
+        """
+        res = []
+        _fields = []
+        _fields.extend(fields)
+        model = get_model_information(cls.registry, cls.__registry_name__)
+        while _fields:
+            field = _fields.pop()
+            if not isinstance(field, str):
+                field = field.name
+
+            if field in res:
+                continue
+
+            _field = model[field]
+            res.append(field)
+            if isinstance(_field, (Column, FakeColumn)):
+                _fields.extend(x
+                               for x, y in model.items()
+                               if (isinstance(y, RelationShip) and
+                                   not isinstance(y, Many2Many))
+                               for mapper in y.column_names
+                               if mapper.attribute_name == field)
+            elif (isinstance(_field, RelationShip) and
+                  not isinstance(_field, Many2Many)):
+                for mapper in _field.column_names:
+                    _fields.append(mapper.attribute_name)
+
+        return res
+
 
 def get_model_information(registry, registry_name):
     model = registry.loaded_namespaces_first_step[registry_name]
@@ -440,83 +547,6 @@ class SqlBase(SqlMixin):
 
         return 1 if values else 0
 
-    @classmethod_cache()
-    def find_remote_attribute_to_expire(cls, *fields):
-        res = uniquedict()
-        _fields = []
-        _fields.extend(fields)
-        model = get_model_information(cls.registry, cls.__registry_name__)
-        while _fields:
-            field = _fields.pop()
-            field = field if isinstance(field, str) else field.name
-            _field = model[field]
-
-            if isinstance(_field, (Column, FakeColumn)):
-                _fields.extend(x for x, y in model.items()
-                               if (isinstance(y, RelationShip) and
-                                   not isinstance(y, Many2Many))
-                               for mapper in y.column_names
-                               if mapper.attribute_name == field)
-                if isinstance(_field, Column) and _field.foreign_key:
-                    rmodel = cls.registry.loaded_namespaces_first_step[
-                        _field.foreign_key.model_name]
-                    for rc in [x for x, y in rmodel.items()
-                               if isinstance(y, RelationShip)
-                               for mapper in y.remote_columns
-                               if mapper.attribute_name == field]:
-                        rfield = rmodel[rc]
-                        if isinstance(rfield, FakeRelationShip):
-                            res.add_in_res(rfield.mapper.attribute_name, [rc])
-                        elif (isinstance(rfield, RelationShip) and
-                              'backref' in rfield.kwargs):
-                            res.add_in_res(
-                                rfield.kwargs['backref'][0], [rc])
-
-            elif (isinstance(_field, RelationShip) and
-                  not isinstance(_field, Many2Many) and
-                  'backref' in _field.kwargs):
-                res.add_in_res(field, [_field.kwargs['backref'][0]])
-            elif isinstance(_field, FakeRelationShip):
-                res.add_in_res(field, [_field.mapper.attribute_name])
-
-        return res
-
-    @classmethod_cache()
-    def find_relationship(cls, *fields):
-        """ Find column and relation ship link with the column or relationship
-        passed in fields.
-
-        :param \*fields: lists of the attribute name
-        :rtype: list of the attribute name of the attribute and relation ship
-        """
-        res = []
-        _fields = []
-        _fields.extend(fields)
-        model = get_model_information(cls.registry, cls.__registry_name__)
-        while _fields:
-            field = _fields.pop()
-            if not isinstance(field, str):
-                field = field.name
-
-            if field in res:
-                continue
-
-            _field = model[field]
-            res.append(field)
-            if isinstance(_field, (Column, FakeColumn)):
-                _fields.extend(x
-                               for x, y in model.items()
-                               if (isinstance(y, RelationShip) and
-                                   not isinstance(y, Many2Many))
-                               for mapper in y.column_names
-                               if mapper.attribute_name == field)
-            elif (isinstance(_field, RelationShip) and
-                  not isinstance(_field, Many2Many)):
-                for mapper in _field.column_names:
-                    _fields.append(mapper.attribute_name)
-
-        return res
-
     def expire_relationship_mapped(self, mappers):
         """ Expire the objects linked with this object, in function of
         the mappers definition
@@ -530,13 +560,13 @@ class SqlBase(SqlMixin):
                 if field is not None:
                     field.expire(*rfields)
 
-    def refresh(self, *fields):
+    def refresh(self, *fields, with_for_update=None):
         """ Expire and reload all the attribute of the instance
 
         See: http://docs.sqlalchemy.org/en/latest/orm/session_api.html
         #sqlalchemy.orm.session.Session.refresh
         """
-        self.registry.refresh(self, fields)
+        self.registry.refresh(self, fields, with_for_update=with_for_update)
 
     def expunge(self):
         """Expunge the instance in the session"""
@@ -551,6 +581,14 @@ class SqlBase(SqlMixin):
         """
         self.registry.expire(self, fields)
 
+    def flag_modified(self, *fields):
+        """ Flag the attributes as modified
+
+        see: http://docs.sqlalchemy.org/en/latest/orm/session_api.html
+        #sqlalchemy.orm.session.Session.expire
+        """
+        self.registry.flag_modified(self, fields)
+
     def delete(self, byquery=False, flush=True):
         """ Call the SqlAlchemy Query.delete method on the instance of the
         model::
