@@ -3,12 +3,15 @@
 #    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2019 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
+#    Copyright (C) 2020 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
+from base64 import b64encode, b64decode
 from .field import Field, FieldException
-from .mapper import ModelAttributeAdapter
+from .mapper import ModelAttributeAdapter, ModelAttribute
 from sqlalchemy.schema import Sequence as SA_Sequence, Column as SA_Column
 from sqlalchemy import types, CheckConstraint
 from sqlalchemy_utils.types.color import ColorType
@@ -19,10 +22,13 @@ from sqlalchemy_utils.types.url import URLType
 from sqlalchemy_utils.types.phone_number import PhoneNumberType
 from sqlalchemy_utils.types.email import EmailType
 from sqlalchemy_utils.types.scalar_coercible import ScalarCoercible
-from datetime import datetime, date
+from sqlalchemy_utils import JSONType
+from datetime import datetime, date, timedelta
 from dateutil.parser import parse
 from inspect import ismethod
 from anyblok.config import Configuration
+from .common import sgdb_in
+from json import dumps, loads
 import time
 import pytz
 import decimal
@@ -46,8 +52,18 @@ logger = getLogger(__name__)
 
 
 def wrap_default(registry, namespace, default_val):
+    """Return default wrapper
 
+    :param registry: the current registry
+    :param namespace: the namespace of the model
+    :param default_val:
+    :return: default wrapper
+    """
     def wrapper():
+        """Return wrapper
+
+        :return: default val
+        """
         Model = registry.get(namespace)
         if hasattr(Model, default_val):
             func = getattr(Model, default_val)
@@ -56,18 +72,20 @@ def wrap_default(registry, namespace, default_val):
                     if default_val not in Model.loaded_fields:
                         return func()
                     else:
-                        logger.warn("On a Model %r the attribute %r is "
-                                    "declared as a default value, a field "
-                                    "with the same name exist" % (namespace,
-                                                                  default_val))
+                        logger.warning(
+                            "The attribute %r is already declared as a default "
+                            "value on the Model %r, a field with the same name "
+                            "already exists" % (default_val, namespace))
                 else:
-                    logger.warn("On a Model %r the attribute %r is declared "
-                                "as a default value, a column with the same "
-                                "name exist" % (namespace, default_val))
+                    logger.warning(
+                        "The attribute %r is already declared as a default "
+                        "value on the Model %r, a column with the same name "
+                        "already exists" % (default_val, namespace))
             else:
-                logger.warn("On a Model %r the attribute %r is declared as a "
-                            "default value, a instance method with the same "
-                            "name exist" % (namespace, default_val))
+                logger.warning(
+                    "The attribute %r is already declared as a default "
+                    "value on the Model %r, an instance method with the same "
+                    "name already exists" % (default_val, namespace))
 
         return default_val
 
@@ -80,17 +98,64 @@ class ColumnDefaultValue:
         self.callable = callable
 
     def get_default_callable(self, registry, namespace, fieldname, properties):
+        """Get default callable
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :param fieldname: the fieldname of the model
+        :param properties: the properties of the model
+        :return: default callable
+        """
         return self.callable(registry, namespace, fieldname, properties)
 
 
+class CompareType:
+
+    comparators = []
+
+    @classmethod
+    def default_comparator(cls, col1, type1, col2, type2):
+        if type1.__class__ is not type2.__class__:
+            raise FieldException(
+                "You can't add a foreign key using columns with different "
+                "types {model1!s}.{col1!s}` pointing to `{model2!s}.{col2!s}` "
+                "have different types  {type1!r} -> {type2!r}".format(
+                    model1=col1.model_name,
+                    col1=col1.attribute_name,
+                    model2=col2.model_name,
+                    col2=col2.attribute_name,
+                    type1=type1.__class__,
+                    type2=type2.__class__
+                )
+            )
+
+    @classmethod
+    def add_comparator(cls, type1, type2):
+
+        def wrapper(funct):
+            cls.comparators.append((type1, type2, funct))
+            return funct
+
+        return wrapper
+
+    @classmethod
+    def validate(cls, col1, type1, col2, type2):
+        for (cls1, cls2, funct) in cls.comparators:
+            if type1.__class__ is cls1 and type2.__class__ is cls2:
+                funct(col1, type1, col2, type2)
+                return
+
+        cls.default_comparator(col1, type1, col2, type2)
+
+
 class NoDefaultValue:
     pass
 
 
 class Column(Field):
-    """ Column class
+    """Column class
 
-    This class can't be instanciated
+    This class can't be instantiated
     """
 
     use_hybrid_property = True
@@ -99,7 +164,7 @@ class Column(Field):
     type = None
 
     def __init__(self, *args, **kwargs):
-        """ Initialize the column
+        """Initialize the column
 
         :param label: label of this field
         :type label: str
@@ -129,6 +194,10 @@ class Column(Field):
         super(Column, self).__init__(*args, **kwargs)
 
     def autodoc_get_properties(self):
+        """Return properties list for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Column, self).autodoc_get_properties()
         res['foreign_key'] = self.foreign_key
         res['DB column name'] = self.db_column_name
@@ -143,12 +212,27 @@ class Column(Field):
         ('is crypted', False),
     ))
 
-    def native_type(cls):
-        """ Return the native SqlAlchemy type """
-        return cls.sqlalchemy_type
+    def native_type(self, registry):
+        """Return the native SqlAlchemy type
 
-    def format_foreign_key(self, registry, args, kwargs):
+        :param registry:
+        :rtype: sqlalchemy native type
+        """
+        return self.sqlalchemy_type
+
+    def format_foreign_key(self, registry, namespace, fieldname, args, kwargs):
+        """Format a foreign key
+
+        :param registry: the current registry
+        :param args:
+        :param kwargs:
+        :return:
+        """
         if self.foreign_key:
+            CompareType.validate(
+                ModelAttribute(namespace, fieldname), self,
+                self.foreign_key, self.foreign_key.get_type(registry)
+            )
             args = args + (self.foreign_key.get_fk(registry),)
             kwargs['info'].update({
                 'foreign_key': self.foreign_key.get_fk_name(registry),
@@ -159,7 +243,7 @@ class Column(Field):
 
     def get_sqlalchemy_mapping(self, registry, namespace, fieldname,
                                properties):
-        """ Return the instance of the real field
+        """Return the instance of the real field
 
         :param registry: current registry
         :param namespace: name of the model
@@ -172,7 +256,9 @@ class Column(Field):
         kwargs = self.kwargs.copy()
         if 'info' not in kwargs:
             kwargs['info'] = {}
-        args = self.format_foreign_key(registry, args, kwargs)
+        args = self.format_foreign_key(
+            registry, namespace, fieldname, args, kwargs)
+
         kwargs['info']['label'] = self.label
         if self.sequence:
             args = (self.sequence,) + args
@@ -192,7 +278,8 @@ class Column(Field):
             else:
                 kwargs['default'] = self.default_val
 
-        sqlalchemy_type = self.sqlalchemy_type
+        sqlalchemy_type = self.native_type(registry)
+
         if self.encrypt_key:
             encrypt_key = self.format_encrypt_key(registry, namespace)
             sqlalchemy_type = EncryptedType(sqlalchemy_type, encrypt_key)
@@ -200,15 +287,25 @@ class Column(Field):
         return SA_Column(db_column_name, sqlalchemy_type, *args, **kwargs)
 
     def format_encrypt_key(self, registry, namespace):
+        """Format and return the encyption key
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :return: encrypt key
+        """
         encrypt_key = self.encrypt_key
         if encrypt_key is True:
             encrypt_key = Configuration.get('default_encrypt_key')
 
         if not encrypt_key:
-            raise FieldException("No encrypt_key define in the "
+            raise FieldException("No encrypt_key defined in the "
                                  "configuration")
 
         def wrapper():
+            """Return encrypt_key wrapper
+
+            :return:
+            """
             Model = registry.get(namespace)
             if hasattr(Model, encrypt_key):
                 func = getattr(Model, encrypt_key)
@@ -222,7 +319,7 @@ class Column(Field):
         return wrapper
 
     def must_be_declared_as_attr(self):
-        """ Return True if the column have a foreign key to a remote column """
+        """Return True if the column have a foreign key to a remote column """
         if self.foreign_key is not None:
             return True
 
@@ -230,7 +327,7 @@ class Column(Field):
 
 
 class Integer(Column):
-    """ Integer column
+    """Integer column
 
     ::
 
@@ -255,7 +352,7 @@ class Integer(Column):
 
 
 class BigInteger(Column):
-    """ Big integer column
+    """Big integer column
 
     ::
 
@@ -273,7 +370,7 @@ class BigInteger(Column):
 
 
 class Boolean(Column):
-    """ Boolean column
+    """Boolean column
 
     ::
 
@@ -291,7 +388,7 @@ class Boolean(Column):
 
 
 class Float(Column):
-    """ Float column
+    """Float column
 
     ::
 
@@ -308,8 +405,15 @@ class Float(Column):
     sqlalchemy_type = types.Float
 
 
+"""
+    Added *process_result_value* at the class *DECIMAL*, because
+    this method is necessary for encrypt the column
+"""
+types.DECIMAL.process_result_value = lambda self, value, dialect: value
+
+
 class Decimal(Column):
-    """ Decimal column
+    """Decimal column
 
     ::
 
@@ -327,15 +431,31 @@ class Decimal(Column):
     sqlalchemy_type = types.DECIMAL
 
     def setter_format_value(self, value):
+        """Format the given value to decimal if needed
+
+        :param value:
+        :return: decimal value
+        """
         if value is not None:
-            if not isinstance(value, decimal.Decimal):
+            if self.encrypt_key:
+                value = str(value)
+            elif not isinstance(value, decimal.Decimal):
                 value = decimal.Decimal(value)
 
         return value
 
+    def getter_format_value(self, value):
+        if value is None:
+            return None
+
+        if self.encrypt_key:
+            value = decimal.Decimal(value)
+
+        return value
+
 
 class Date(Column):
-    """ Date column
+    """Date column
 
     ::
 
@@ -354,6 +474,11 @@ class Date(Column):
 
 
 def convert_string_to_datetime(value):
+    """Convert a given value to datetime
+
+    :param value:
+    :return: datetime value
+    """
     if value is None:
         return None
     elif isinstance(value, datetime):
@@ -367,6 +492,12 @@ def convert_string_to_datetime(value):
 
 
 def add_timezone_on_datetime(dt, default_timezone):
+    """Convert a datetime considering the default timezone
+
+    :param dt:
+    :param default_timezone:
+    :return:
+    """
     if dt is not None:
         if dt.tzinfo is None:
             dt = default_timezone.localize(dt)
@@ -402,7 +533,7 @@ class DateTimeType(types.TypeDecorator):
 
 
 class DateTime(Column):
-    """ DateTime column
+    """DateTime column
 
     ::
 
@@ -435,17 +566,54 @@ class DateTime(Column):
         super(DateTime, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
+        """Return converted and formatted value
+
+        :param value:
+        :return:
+        """
         value = convert_string_to_datetime(value)
         return add_timezone_on_datetime(value, self.default_timezone)
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Column, self).autodoc_get_properties()
         res['is auto updated'] = self.auto_update
+        if self.default_timezone:
+            res['default timezone'] = self.default_timezone
+
         return res
 
 
+class TimeStamp(DateTime):
+    """ TimeStamp column
+
+    ::
+
+        from datetime import datetime
+        from anyblok.declarations import Declarations
+        from anyblok.column import DateTime
+
+
+        @Declarations.register(Declarations.Model)
+        class Test:
+
+            x = TimeStamp(default=datetime.now)
+
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(TimeStamp, self).__init__(*args, **kwargs)
+        self.sqlalchemy_type = types.TIMESTAMP(timezone=True)
+
+    def getter_format_value(self, value):
+        return convert_string_to_datetime(value)
+
+
 class Time(Column):
-    """ Time column
+    """Time column
 
     ::
 
@@ -464,7 +632,7 @@ class Time(Column):
 
 
 class Interval(Column):
-    """ Datetime interval column
+    """Datetime interval column
 
     ::
 
@@ -481,6 +649,26 @@ class Interval(Column):
     """
     sqlalchemy_type = types.Interval
 
+    def native_type(self, registry):
+        if self.encrypt_key:
+            return types.VARCHAR(1024)
+
+        return self.sqlalchemy_type
+
+    def setter_format_value(self, value):
+        if self.encrypt_key:
+            value = dumps({
+                x: getattr(value, x)
+                for x in ['days', 'seconds', 'microseconds']})
+
+        return value
+
+    def getter_format_value(self, value):
+        if self.encrypt_key:
+            value = timedelta(**loads(value))
+
+        return value
+
 
 class StringType(types.TypeDecorator):
 
@@ -497,7 +685,7 @@ class StringType(types.TypeDecorator):
 
 
 class String(Column):
-    """ String column
+    """String column
 
     ::
 
@@ -513,20 +701,71 @@ class String(Column):
     """
     def __init__(self, *args, **kwargs):
         self.size = kwargs.pop('size', 64)
-        if 'type_' in kwargs:
-            del kwargs['type_']
-
+        kwargs.pop('type_', None)
         self.sqlalchemy_type = StringType(self.size)
         super(String, self).__init__(*args, **kwargs)
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(String, self).autodoc_get_properties()
         res['size'] = self.size
         return res
 
 
+class Enum(Column):
+    """Enum column
+
+    ::
+
+        from anyblok.declarations import Declarations
+        from anyblok.column import Enum
+        import enum
+
+
+        class MyEnumClass(enum.Enum):
+            one = 1
+            two = 2
+            three = 3
+
+
+        @Declarations.register(Declarations.Model)
+        class Test:
+
+            x = Enum(enum_cls=MyEnumClass, default='test')
+
+    enum_cls should be an enum class
+    """
+    def __init__(self, *args, **kwargs):
+        self.enum_cls = kwargs.pop('enum_cls')
+        self.sqlalchemy_type = types.Enum(self.enum_cls)
+        super(Enum, self).__init__(*args, **kwargs)
+
+    def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
+        res = super(Enum, self).autodoc_get_properties()
+        res['enum_cls'] = repr(self.enum_cls)
+        return res
+
+
+class MsSQLPasswordType(PasswordType):
+    impl = types.VARCHAR(1024)
+
+    def load_dialect_impl(self, dialect):
+        if dialect.name == 'mssql':
+            # Use a BLOB type for sqlite
+            return dialect.type_descriptor(types.VARCHAR(self.length))
+
+        return super(MsSQLPasswordType, self).load_dialect_impl(dialect)
+
+
 class Password(Column):
-    """ String column
+    """String column
 
     ::
 
@@ -558,26 +797,42 @@ class Password(Column):
             ==> 0
     """
     def __init__(self, *args, **kwargs):
-        size = kwargs.pop('size', 64)
+        self.size = kwargs.pop('size', 64)
         crypt_context = kwargs.pop('crypt_context', {})
         self.crypt_context = crypt_context
-
-        if 'type_' in kwargs:
-            del kwargs['type_']
+        kwargs.pop('type_', None)
 
         if 'foreign_key' in kwargs:
             raise FieldException("Column Password can not have a foreign key")
 
-        self.sqlalchemy_type = PasswordType(max_length=size, **crypt_context)
+        self.sqlalchemy_type = PasswordType(
+            max_length=self.size, **crypt_context)
         super(Password, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
+        """Return formatted value
+
+        :param value:
+        :return:
+        """
         value = self.sqlalchemy_type.context.encrypt(value).encode('utf8')
         value = SAU_PWD(value, context=self.sqlalchemy_type.context)
         return value
 
+    def native_type(self, registry):
+        """ Return the native SqlAlchemy type """
+        if sgdb_in(registry.engine, ['MsSQL']):
+            return MsSQLPasswordType(max_length=self.size, **self.crypt_context)
+
+        return self.sqlalchemy_type
+
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Password, self).autodoc_get_properties()
+        res['size'] = self.size
         res['Crypt context'] = self.crypt_context
         return res
 
@@ -597,7 +852,7 @@ class TextType(types.TypeDecorator):
 
 
 class Text(Column):
-    """ Text column
+    """Text column
 
     ::
 
@@ -615,12 +870,16 @@ class Text(Column):
 
 
 class StrSelection(str):
-    """ Class representing the data of one column Selection """
+    """Class representing the data of one column Selection """
     selections = {}
     registry = None
     namespace = None
 
     def get_selections(self):
+        """Return a dict of selections
+
+        :return: selections dict
+        """
         if isinstance(self.selections, dict):
             return self.selections
         if isinstance(self.selections, str):
@@ -628,17 +887,25 @@ class StrSelection(str):
             return dict(getattr(m, self.selections)())
 
     def validate(self):
+        """Validate if the key is in the selections
+
+        :return: True or False
+        """
         a = super(StrSelection, self).__str__()
         return a in self.get_selections().keys()
 
     @property
     def label(self):
+        """Return the label corresponding to the selection key
+
+        :return:
+        """
         a = super(StrSelection, self).__str__()
         return self.get_selections()[a]
 
 
 class SelectionType(types.TypeDecorator):
-    """ Generic type for Column Selection """
+    """Generic type for Column Selection """
 
     impl = types.String
 
@@ -682,7 +949,7 @@ class SelectionType(types.TypeDecorator):
 
 
 class Selection(Column):
-    """ Selection column
+    """Selection column
 
     ::
 
@@ -702,30 +969,42 @@ class Selection(Column):
     """
     def __init__(self, *args, **kwargs):
         self.selections = tuple()
-        self.size = 64
         if 'selections' in kwargs:
             self.selections = kwargs.pop('selections')
 
-        if 'size' in kwargs:
-            self.size = kwargs.pop('size')
-
+        self.size = kwargs.pop('size', 64)
         self.sqlalchemy_type = 'tmp value for assert'
 
         super(Selection, self).__init__(*args, **kwargs)
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Selection, self).autodoc_get_properties()
         res['selections'] = self.selections
         res['size'] = self.size
         return res
 
     def getter_format_value(self, value):
+        """Return formatted value
+
+        :param value:
+        :return:
+        """
         if value is None:
             return None
 
         return self.sqlalchemy_type.python_type(value)
 
     def setter_format_value(self, value):
+        """Return value or raise exception if the given value is invalid
+
+        :param value:
+        :exception FieldException
+        :return:
+        """
         if value is not None:
             val = self.sqlalchemy_type.python_type(value)
             if not val.validate():
@@ -736,21 +1015,36 @@ class Selection(Column):
 
     def get_sqlalchemy_mapping(self, registry, namespace, fieldname,
                                properties):
+        """Return sqlalchmy mapping
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :param fieldname: the fieldname of the model
+        :param properties: the properties of the model
+        :return: instance of the real field
+        """
         self.sqlalchemy_type = SelectionType(
             self.selections, self.size, registry=registry, namespace=namespace)
         return super(Selection, self).get_sqlalchemy_mapping(
             registry, namespace, fieldname, properties)
 
     def update_description(self, registry, model, res):
+        """Update model description
+
+        :param registry: the current registry
+        :param model:
+        :param res:
+        """
         super(Selection, self).update_description(registry, model, res)
         sqlalchemy_type = SelectionType(
             self.selections, self.size, registry=registry, namespace=model)
         values = sqlalchemy_type._StrSelection().get_selections()
         res['selections'] = [(k, v) for k, v in values.items()]
 
-    def must_be_duplicate_before_added(self):
-        """ Return True because the field selection in a mixin must be copied
-        else the selection method can be wrond
+    def must_be_copied_before_declaration(self):
+        """Return True if selections is an instance of str.
+        In the case of the field selection is a mixin, it must be copied or the
+        selection method can fail
         """
         if isinstance(self.selections, str):
             return True
@@ -758,13 +1052,34 @@ class Selection(Column):
             return False
 
     def update_properties(self, registry, namespace, fieldname, properties):
+        """Update column properties
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :param fieldname: the fieldname of the model
+        :param properties: the properties of the model
+        """
         super(Selection, self).update_properties(registry, namespace,
                                                  fieldname, properties)
         self.fieldname = fieldname
         properties['add_in_table_args'].append(self)
 
-    def update_table_args(self, Model):
-        """return check constraint to limit the value"""
+    def update_table_args(self, registry, Model):
+        """Return check constraints to limit the value
+
+        :param registry:
+        :param Model:
+        :return: list of checkConstraint
+        """
+        if self.encrypt_key:
+            # dont add constraint because the state is crypted and nobody
+            # can add new entry
+            return []
+
+        if sgdb_in(registry.engine, ['MariaDB', 'MsSQL']):
+            # No check constraint in MariaDB
+            return []
+
         selections = self.sqlalchemy_type.selections
         if isinstance(selections, dict):
             enum = selections.keys()
@@ -794,8 +1109,15 @@ class Selection(Column):
         return []
 
 
+"""
+    Added *process_result_value* at the class *JSON*, because
+    this method is necessary for encrypt the column
+"""
+types.JSON.process_result_value = lambda self, value, dialect: value
+
+
 class Json(Column):
-    """ JSON column
+    """JSON column
 
     ::
 
@@ -811,9 +1133,31 @@ class Json(Column):
     """
     sqlalchemy_type = types.JSON(none_as_null=True)
 
+    def native_type(self, registry):
+        """ Return the native SqlAlchemy type """
+        if sgdb_in(registry.engine, ['MariaDB', 'MsSQL']):
+            return JSONType
+
+        return self.sqlalchemy_type
+
+    def setter_format_value(self, value):
+        if self.encrypt_key:
+            value = dumps(value)
+
+        return value
+
+    def getter_format_value(self, value):
+        if value is None:
+            return None
+
+        if self.encrypt_key:
+            value = loads(value)
+
+        return value
+
 
 class LargeBinary(Column):
-    """ Large binary column
+    """Large binary column
 
     ::
 
@@ -833,9 +1177,27 @@ class LargeBinary(Column):
     """
     sqlalchemy_type = types.LargeBinary
 
+    def native_type(self, registry):
+        if self.encrypt_key:
+            return types.Text
+
+        return self.sqlalchemy_type
+
+    def setter_format_value(self, value):
+        if self.encrypt_key:
+            value = b64encode(value).decode('utf-8')
+
+        return value
+
+    def getter_format_value(self, value):
+        if self.encrypt_key:
+            value = b64decode(value.encode('utf-8'))
+
+        return value
+
 
 class Sequence(String):
-    """ Sequence column
+    """Sequence column
 
     ::
 
@@ -858,17 +1220,29 @@ class Sequence(String):
         kwargs['default'] = ColumnDefaultValue(self.wrap_default)
 
         self.code = kwargs.pop('code') if 'code' in kwargs else None
+        self.start = kwargs.pop('start', 1)
         self.formater = kwargs.pop(
             'formater') if 'formater' in kwargs else None
 
         super(Sequence, self).__init__(*args, **kwargs)
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Sequence, self).autodoc_get_properties()
         res['formater'] = self.formater
         return res
 
     def wrap_default(self, registry, namespace, fieldname, properties):
+        """Return default wrapper
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :param fieldname: the fieldname of the model
+        :return:
+        """
         if not hasattr(registry, '_need_sequence_to_create_if_not_exist'):
             registry._need_sequence_to_create_if_not_exist = []
         elif registry._need_sequence_to_create_if_not_exist is None:
@@ -876,9 +1250,13 @@ class Sequence(String):
 
         code = self.code if self.code else "%s=>%s" % (namespace, fieldname)
         registry._need_sequence_to_create_if_not_exist.append(
-            {'code': code, 'formater': self.formater})
+            {'code': code, 'formater': self.formater, 'start': self.start})
 
         def default_value():
+            """Return next sequence value
+
+            :return:
+            """
             return registry.System.Sequence.nextvalBy(code=code)
 
         return default_value
@@ -886,7 +1264,7 @@ class Sequence(String):
 
 class Color(Column):
     """Color column.
-    `See coulour pakage <https://pypi.python.org/pypi/colour/>`_
+    `See colour package on pypi <https://pypi.python.org/pypi/colour/>`_
 
     ::
 
@@ -901,28 +1279,34 @@ class Color(Column):
 
     """
     def __init__(self, *args, **kwargs):
-        max_length = kwargs.pop('size', 20)
-
-        if 'type_' in kwargs:
-            del kwargs['type_']
-
+        self.max_length = max_length = kwargs.pop('size', 20)
+        kwargs.pop('type_', None)
         self.sqlalchemy_type = ColorType(max_length)
         super(Color, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
+        """Format the given value
+
+        :param value:
+        :return:
+        """
         if isinstance(value, str):
             value = self.sqlalchemy_type.python_type(value)
 
         return value
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Color, self).autodoc_get_properties()
         res['size'] = self.max_length
         return res
 
 
 class UUID(Column):
-    """ UUID column
+    """UUID column
 
     ::
 
@@ -948,6 +1332,10 @@ class UUID(Column):
         super(UUID, self).__init__(*args, **kwargs)
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(UUID, self).autodoc_get_properties()
         res['binary'] = self.binary
         res['native'] = self.native
@@ -955,7 +1343,7 @@ class UUID(Column):
 
 
 class URL(Column):
-    """ URL column
+    """URL column
 
     ::
 
@@ -972,6 +1360,11 @@ class URL(Column):
     sqlalchemy_type = URLType
 
     def setter_format_value(self, value):
+        """Return formatted url value
+
+        :param value:
+        :return:
+        """
         from furl import furl
 
         if value is not None:
@@ -982,7 +1375,7 @@ class URL(Column):
 
 
 class PhoneNumber(Column):
-    """ PhoneNumber column
+    """PhoneNumber column
 
     ::
 
@@ -1002,28 +1395,43 @@ class PhoneNumber(Column):
     def __init__(self, region='FR', max_length=20, *args, **kwargs):
         self.region = region
         self.max_length = max_length
-        if 'type_' in kwargs:
-            del kwargs['type_']
+        kwargs.pop('type_', None)
 
         self.sqlalchemy_type = PhoneNumberType(
             region=region, max_length=max_length)
         super(PhoneNumber, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
-        if isinstance(value, str):
-            value = self.sqlalchemy_type.python_type(value)
+        """Return formatted phone number value
+
+        :param value:
+        :return:
+        """
+        if value and isinstance(value, str):
+            value = self.sqlalchemy_type.python_type(value, self.region)
 
         return value
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(PhoneNumber, self).autodoc_get_properties()
         res['region'] = self.region
         res['max_length'] = self.max_length
         return res
 
 
+"""
+    Added *process_result_value* at the class *EmailType*, because
+    this method is necessary for encrypt the column
+"""
+EmailType.process_result_value = lambda self, value, dialect: value
+
+
 class Email(Column):
-    """ Email column
+    """Email column
 
     ::
 
@@ -1039,13 +1447,18 @@ class Email(Column):
     sqlalchemy_type = EmailType
 
     def setter_format_value(self, value):
+        """Return formatted email value
+
+        :param value:
+        :return:
+        """
         if value is not None:
             return value.lower()
         return value
 
 
 class CountryType(types.TypeDecorator, ScalarCoercible):
-    """ Generic type for Column Country """
+    """Generic type for Column Country """
 
     impl = types.Unicode(3)
     python_type = python_pycountry_type
@@ -1098,28 +1511,59 @@ class Country(Column):
         super(Country, self).__init__(*args, **kwargs)
 
     def setter_format_value(self, value):
+        """Return formatted country value
+
+        :param value:
+        :return:
+        """
         if value and not isinstance(value, self.sqlalchemy_type.python_type):
-            try:
-                value = pycountry.countries.get(**{self.mode: value})
-            except LookupError:
-                value = pycountry.countries.lookup(value)
+            value = pycountry.countries.get(
+                **{
+                    self.mode: value,
+                    'default': pycountry.countries.lookup(value)
+                })
 
         return value
 
     def autodoc_get_properties(self):
+        """Return properties for autodoc
+
+        :return: autodoc properties
+        """
         res = super(Country, self).autodoc_get_properties()
         res['mode'] = self.mode
         res['choices'] = self.choices
         return res
 
     def update_properties(self, registry, namespace, fieldname, properties):
+        """Update column properties
+
+        :param registry: the current registry
+        :param namespace: the namespace of the model
+        :param fieldname: the fieldname of the model
+        :param properties: the properties of the model
+        """
         super(Country, self).update_properties(registry, namespace,
                                                fieldname, properties)
         self.fieldname = fieldname
         properties['add_in_table_args'].append(self)
 
-    def update_table_args(self, Model):
-        """return check constraint to limit the value"""
+    def update_table_args(self, registry, Model):
+        """Return check constraints to limit the value
+
+        :param registry:
+        :param Model:
+        :return: list of checkConstraint
+        """
+        if self.encrypt_key:
+            # dont add constraint because the state is crypted and nobody
+            # can add new entry
+            return []
+
+        if sgdb_in(registry.engine, ['MariaDB', 'MsSQL']):
+            # No Check constraint in MariaDB
+            return []
+
         enum = [country.alpha_3 for country in pycountry.countries]
         constraint = """"%s" in ('%s')""" % (self.fieldname, "', '".join(enum))
         enum.sort()
@@ -1127,3 +1571,45 @@ class Country(Column):
         key.update(str(enum).encode('utf-8'))
         name = self.fieldname + '_' + key.hexdigest() + '_types'
         return [CheckConstraint(constraint, name=name)]
+
+
+@CompareType.add_comparator(String, String)
+@CompareType.add_comparator(String, Selection)
+@CompareType.add_comparator(String, Sequence)
+def compare_strings(col1, type1, col2, type2):
+    if type1.size != type2.size:
+        raise FieldException(
+            "You can't add a foreign key using based String columns with "
+            "different size `{model1!s}.{col1!s}` pointing to "
+            "`{model2!s}.{col2!s}` have different sizes  {type1!r}({size1:d}) "
+            "-> ({type2!r}){size2:d}".format(
+                model1=col1.model_name,
+                col1=col1.attribute_name,
+                model2=col2.model_name,
+                col2=col2.attribute_name,
+                type1=type1.__class__,
+                type2=type2.__class__,
+                size1=type1.size,
+                size2=type2.size
+            )
+        )
+
+
+@CompareType.add_comparator(String, Color)
+def compare_string_to_color(col1, type1, col2, type2):
+    if type1.size != type2.max_length:
+        raise FieldException(
+            "You can't add a foreign key using based String columns with "
+            "different size `{model1!s}.{col1!s}` pointing to "
+            "`{model2!s}.{col2!s}` have different sizes  {type1!r}({size1:d}) "
+            "-> ({type2!r}){size2:d}".format(
+                model1=col1.model_name,
+                col1=col1.attribute_name,
+                model2=col2.model_name,
+                col2=col2.attribute_name,
+                type1=type1.__class__,
+                type2=type2.__class__,
+                size1=type1.size,
+                size2=type2.max_length
+            )
+        )
