@@ -2,6 +2,8 @@
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2019 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -11,60 +13,17 @@ from anyblok import Declarations
 from anyblok.field import Field, FieldException
 from anyblok.relationship import RelationShip
 from anyblok.column import Column
-from sqlalchemy.ext.compiler import compiles
-from sqlalchemy.schema import DDLElement
-from sqlalchemy.sql import table
-from sqlalchemy.orm import Query, mapper, synonym
 from sqlalchemy import inspection
 from anyblok.common import TypeList
 from copy import deepcopy
 from sqlalchemy.ext.declarative import declared_attr
-from anyblok.mapper import ModelAttribute
+from anyblok.mapper import ModelAttribute, format_schema
 from anyblok.common import anyblok_column_prefix
 from texttable import Texttable
 from .plugins import get_model_plugins
-from .exceptions import ModelException, ViewException
-
-
-class CreateView(DDLElement):
-    def __init__(self, name, selectable):
-        self.name = name
-        self.selectable = selectable
-
-
-class DropView(DDLElement):
-    def __init__(self, name):
-        self.name = name
-
-
-@compiles(CreateView)
-def compile_create_view(element, compiler, **kw):
-    return "CREATE VIEW %s AS %s" % (
-        element.name, compiler.sql_compiler.process(element.selectable))
-
-
-@compiles(DropView)
-def compile_drop_view(element, compiler, **kw):
-    return "DROP VIEW IF EXISTS %s" % (element.name)
-
-
-def has_sql_fields(bases):
-    """ Tells whether the model as field or not
-
-    :param bases: list of Model's Class
-    :rtype: boolean
-    """
-    for base in bases:
-        for p in base.__dict__.keys():
-            try:
-                if hasattr(getattr(base, p), '__class__'):
-                    if Field in getattr(base, p).__class__.__mro__:
-                        return True
-            except FieldException:
-                # field function case already computed
-                return True
-
-    return False
+from .exceptions import ModelException
+from .factory import has_sql_fields, ModelFactory, ViewFactory
+from .common import get_factory
 
 
 def has_sqlalchemy_fields(base):
@@ -130,6 +89,15 @@ def autodoc_fields(declaration_cls, model_cls):
     return table.draw() + '\n\n'
 
 
+def update_factory(kwargs):
+    if 'factory' in kwargs:
+        kwargs['__model_factory__'] = kwargs.pop('factory')
+    elif 'type' in kwargs:
+        kwargs['__model_factory__'] = get_factory(kwargs.pop('type'))
+    elif kwargs.get('is_sql_view'):
+        kwargs['__model_factory__'] = ViewFactory
+
+
 @Declarations.add_declaration_type(isAnEntry=True,
                                    pre_assemble='pre_assemble_callback',
                                    assemble='assemble_callback',
@@ -224,6 +192,7 @@ class Model:
 
         kwargs['__registry_name__'] = _registryname
         kwargs['__tablename__'] = tablename
+        update_factory(kwargs)
 
         RegistryManager.add_entry_in_register(
             'Model', _registryname, cls_, **kwargs)
@@ -254,7 +223,7 @@ class Model:
         if name in properties['loaded_columns']:
             return
 
-        if field.must_be_duplicate_before_added():
+        if field.must_be_copied_before_declaration():
             field = deepcopy(field)
 
         attr_name = name
@@ -347,7 +316,10 @@ class Model:
             return registry.loaded_namespaces_first_step[namespace]
 
         bases = []
-        properties = {'__depends__': set()}
+        properties = {
+            '__depends__': set(),
+            '__db_schema__': format_schema(None, namespace)
+        }
         ns = registry.loaded_registries[namespace]
 
         for b in ns['bases']:
@@ -359,6 +331,7 @@ class Model:
 
                 ps = cls.load_namespace_first_step(registry,
                                                    b_ns.__registry_name__)
+                ps = ps.copy()
                 ps.update(properties)
                 properties.update(ps)
 
@@ -369,6 +342,10 @@ class Model:
                 if p not in properties:
                     properties[p] = f
 
+            if hasattr(b, '__db_schema__'):
+                properties['__db_schema__'] = format_schema(b.__db_schema__,
+                                                            namespace)
+
         if '__tablename__' in ns['properties']:
             properties['__tablename__'] = ns['properties']['__tablename__']
 
@@ -419,16 +396,7 @@ class Model:
         properties['loaded_columns'] = []
         properties['hybrid_property_columns'] = []
         properties['loaded_fields'] = {}
-        if properties['is_sql_view']:
-            bases.extend([x for x in registry.loaded_cores['SqlViewBase']])
-        elif has_sql_fields(bases):
-            bases.extend([x for x in registry.loaded_cores['SqlBase']])
-            bases.append(registry.declarativebase)
-        else:
-            # remove tablename to inherit from a sqlmodel
-            del properties['__tablename__']
-
-        bases.extend([x for x in registry.loaded_cores['Base']])
+        properties['__model_factory__'].insert_core_bases(bases, properties)
 
     @classmethod
     def declare_all_fields(cls, registry, namespace, bases, properties,
@@ -495,12 +463,15 @@ class Model:
         bases = TypeList(cls, registry, namespace, transformation_properties)
         ns = registry.loaded_registries[namespace]
         properties = ns['properties'].copy()
+        first_step = registry.loaded_namespaces_first_step[namespace]
+        properties['__depends__'] = first_step['__depends__']
+        properties['__db_schema__'] = first_step.get('__db_schema__', None)
 
         registry.call_plugins('initialisation_tranformation_properties',
                               properties, transformation_properties)
 
-        if 'is_sql_view' not in properties:
-            properties['is_sql_view'] = False
+        properties['__model_factory__'] = properties.get(
+            '__model_factory__', ModelFactory)(registry)
 
         cls.apply_inheritance_base(registry, namespace, ns, bases,
                                    realregistryname, properties,
@@ -522,14 +493,9 @@ class Model:
             bases.append(registry.registry_base)
             cls.insert_in_bases(registry, namespace, bases,
                                 transformation_properties, properties)
-            if properties['is_sql_view']:
-                cls.replace_properties_by_synonym(properties)
-                bases = [type(modelname, tuple(bases), properties)]
-                if properties['is_sql_view']:
-                    cls.apply_view(namespace, tablename, bases[0], registry,
-                                   properties)
-            else:
-                bases = [type(modelname, tuple(bases), properties)]
+            bases = [
+                properties['__model_factory__'].build_model(
+                    modelname, bases, properties)]
 
             properties = {}
             registry.add_in_registry(namespace, bases[0])
@@ -539,61 +505,6 @@ class Model:
 
         return bases, properties
 
-    @classmethod
-    def replace_properties_by_synonym(cls, properties):
-        for field in properties['loaded_columns']:
-            properties[field] = synonym(anyblok_column_prefix + field)
-
-    @classmethod
-    def apply_view(cls, namespace, tablename, base, registry, properties):
-        """ Transform the sqlmodel to view model
-
-        :param namespace: Namespace of the model
-        :param tablename: Name od the table of the model
-        :param base: Model cls
-        :param registry: current registry
-        :param properties: properties of the model
-        :exception: MigrationException
-        :exception: ViewException
-        """
-        if hasattr(base, '__view__'):
-            view = base.__view__
-        elif tablename in registry.loaded_views:
-            view = registry.loaded_views[tablename]
-        else:
-            if not hasattr(base, 'sqlalchemy_view_declaration'):
-                raise ViewException(
-                    "%r.'sqlalchemy_view_declaration' is required to "
-                    "define the query to apply of the view" % namespace)
-
-            view = table(tablename)
-            registry.loaded_views[tablename] = view
-            selectable = getattr(base, 'sqlalchemy_view_declaration')()
-
-            if isinstance(selectable, Query):
-                selectable = selectable.subquery()
-
-            for c in selectable.c:
-                c._make_proxy(view)
-
-            DropView(tablename).execute_at(
-                'before-create', registry.declarativebase.metadata)
-            CreateView(tablename, selectable).execute_at(
-                'after-create', registry.declarativebase.metadata)
-            DropView(tablename).execute_at(
-                'before-drop', registry.declarativebase.metadata)
-
-        pks = [col for col in properties['loaded_columns']
-               if getattr(base, anyblok_column_prefix + col).primary_key]
-
-        if not pks:
-            raise ViewException(
-                "%r have any primary key defined" % namespace)
-
-        pks = [getattr(view.c, x) for x in pks]
-        mapper(base, view, primary_key=pks)
-        setattr(base, '__view__', view)
-
     @classmethod
     def assemble_callback(cls, registry):
         """ Assemble callback is called to assemble all the Model
