@@ -2,15 +2,12 @@
 # This file is a part of the AnyBlok project
 #
 #    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
-from os.path import join
-from os import walk
 from logging import getLogger
-import nose
-
 from sqlalchemy import create_engine, event, MetaData
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker, scoped_session
@@ -24,7 +21,21 @@ from .environment import EnvironmentManager
 from .authorization.query import QUERY_WITH_NO_RESULTS, PostFilteredQuery
 from anyblok.common import anyblok_column_prefix, naming_convention
 from pkg_resources import iter_entry_points
+from .version import parse_version
 from .logging import log
+import sqlalchemy.interfaces
+from sqlalchemy.orm.session import close_all_sessions
+from sqlalchemy.orm.attributes import flag_modified
+
+try:
+    import pyodbc
+    pyodbc.pooling = False
+    PyODBCProgrammingError = pyodbc.ProgrammingError
+except ImportError:
+
+    class PyODBCProgrammingError(Exception):
+        pass
+
 logger = getLogger(__name__)
 
 
@@ -47,6 +58,14 @@ def return_list(entry):
     return entry
 
 
+class DontBeSilly(sqlalchemy.interfaces.PoolListener):
+    def connect(self, dbapi_con, connection_record):
+        cur = dbapi_con.cursor()
+        cur.execute("SET autocommit=0;")
+        cur.execute("SET SESSION sql_mode='TRADITIONAL';")
+        cur = None
+
+
 class RegistryManager:
     """ Manage the global registry
 
@@ -80,7 +99,6 @@ class RegistryManager:
     callback_initialize_entries = {}
     callback_unload_entries = {}
     registries = {}
-    needed_bloks = []
 
     @classmethod
     def has_blok(cls, blok):
@@ -365,7 +383,7 @@ class RegistryManager:
     def has_blok_property(cls, property_):
         """ Return True if the property exists in blok
 
-        :param property\_: name of the property
+        :param property_: name of the property
         """
         blok = EnvironmentManager.get('current_blok')
 
@@ -378,7 +396,7 @@ class RegistryManager:
     def add_or_replace_blok_property(cls, property_, value):
         """ Save the value in the properties
 
-        :param property\_: name of the property
+        :param property_: name of the property
         :param value: the value to save, the type is not important
         """
         blok = EnvironmentManager.get('current_blok')
@@ -388,7 +406,7 @@ class RegistryManager:
     def get_blok_property(cls, property_, default=None):
         """ Return the value in the properties
 
-        :param property\_: name of the property
+        :param property_: name of the property
         :param default: return default If not entry in the property
         """
         blok = EnvironmentManager.get('current_blok')
@@ -398,22 +416,12 @@ class RegistryManager:
     def remove_blok_property(cls, property_):
         """ Remove the property if exist
 
-        :param property\_: name of the property
+        :param property_: name of the property
         """
         blok = EnvironmentManager.get('current_blok')
         if cls.has_blok_property(property_):
             del cls.loaded_bloks[blok]['properties'][property_]
 
-    @classmethod
-    def add_needed_bloks(cls, *bloks):
-        for blok in bloks:
-            if blok not in cls.needed_bloks:
-                cls.needed_bloks.append(blok)
-
-    @classmethod
-    def get_needed_bloks(cls):
-        return cls.needed_bloks
-
 
 class Registry:
     """ Define one registry
@@ -452,9 +460,9 @@ class Registry:
             self.bind = self.engine
             self.unittest_transaction = None
 
-    def init_engine_options(self):
+    def init_engine_options(self, url):
         """Define the options to initialize the engine"""
-        return dict(
+        options = dict(
             echo=Configuration.get('db_echo') or False,
             max_overflow=Configuration.get('db_max_overflow') or 10,
             echo_pool=Configuration.get('db_echo_pool') or False,
@@ -463,15 +471,20 @@ class Registry:
                 'isolation_level',
                 Configuration.get('isolation_level', 'READ_UNCOMMITTED')
             ),
+            listeners=[]
         )
+        if url.drivername.startswith('mysql'):
+            options['listeners'].append(DontBeSilly())
+
+        return options
 
     def init_engine(self, db_name=None):
         """Define the engine
 
         :param db_name: name of the database to link
         """
-        kwargs = self.init_engine_options()
         url = Configuration.get('get_url', get_url)(db_name=db_name)
+        kwargs = self.init_engine_options(url)
         self.rw_engine = create_engine(url, **kwargs)
 
     @property
@@ -498,6 +511,10 @@ class Registry:
         self._sqlalchemy_known_events = []
         self.expire_attributes = {}
 
+        # key = tablename
+        # value = True if all table else list of columns names
+        self.ignore_migration_for = {}
+
     @classmethod
     def db_exists(cls, db_name=None):
         if not db_name:
@@ -551,18 +568,18 @@ class Registry:
             return []
 
         res = []
-        query = """
-            SELECT system_blok.name
-            FROM system_blok
-            WHERE system_blok.state in ('%s')
-            ORDER BY system_blok.order""" % "', '".join(states)
+        query = """SELECT "order", name"""
+        query += " FROM system_blok"
+        query += " WHERE state in ('%s')" % "', '".join(states)
         try:
-            res = self.execute(query).fetchall()
-        except (ProgrammingError, OperationalError):
+            res = self.execute(query, fetchall=True)
+        except (ProgrammingError, OperationalError, PyODBCProgrammingError):
+            # During the first connection the database is empty
             pass
 
         if res:
-            return [x[0] for x in res]
+            res.sort()
+            return [x[1] for x in res]
 
         return []
 
@@ -583,10 +600,6 @@ class Registry:
             if blok not in (toinstall + loaded):
                 toinstall.append(blok)
 
-        for blok in RegistryManager.get_needed_bloks():
-            if blok not in (toinstall + loaded):
-                toinstall.append(blok)
-
         if toinstall and self.withoutautomigration:
             raise RegistryManagerException(
                 "Install modules %r is forbidden with no auto migration "
@@ -813,13 +826,22 @@ class Registry:
         return False
 
     def execute(self, *args, **kwargs):
+        fetchall = kwargs.pop('fetchall', False)
         if self.Session:
-            return self.session.execute(*args, **kwargs)
+            res = self.session.execute(*args, **kwargs)
+            if fetchall:
+                return res.fetchall()
+
+            return res
         else:
             conn = None
             try:
                 conn = self.engine.connect()
-                return conn.execute(*args, **kwargs)
+                res = conn.execute(*args, **kwargs)
+                if fetchall:
+                    return res.fetchall()
+
+                return res
             finally:
                 if conn:
                     conn.close()
@@ -898,6 +920,7 @@ class Registry:
             self.Session = scoped_session(
                 sessionmaker(bind=bind, class_=Session, extension=extension),
                 EnvironmentManager.scoped_function_for_session())
+
             self.nb_query_bases = len(self.loaded_cores['Query'])
             self.nb_session_bases = len(self.loaded_cores['Session'])
             self.apply_session_events()
@@ -952,31 +975,18 @@ class Registry:
             self.assemble_entries()
             self.create_session_factory()
 
-            mustreload = self.apply_model_schema_on_table(
-                blok2install) or mustreload
+            self.apply_model_schema_on_table(blok2install)
+            self.listen_sqlalchemy_known_event()
+            mustreload = self.is_reload_needed() or mustreload
 
         except Exception as e:
             self.close()
             raise e
 
-        test_blok = blok2install and Configuration.get(
-            'test_blok_at_install')
-        selected_bloks = return_list(Configuration.get('selected_bloks'))
-        in_selected_bloks = blok2install in (selected_bloks or [blok2install])
-        unwanted_bloks = return_list(Configuration.get('unwanted_bloks'))
-        not_in_unwanted_bloks = blok2install not in (unwanted_bloks or [])
-
-        if test_blok and in_selected_bloks and not_in_unwanted_bloks:
-            self.System.Blok.load_all()
-            self.run_test(blok2install)
-            if len(toinstall) > 1 or mustreload:
-                self.reload()
-
+        if len(toinstall) > 1 or mustreload:
+            self.reload()
         else:
-            if len(toinstall) > 1 or mustreload:
-                self.reload()
-            else:
-                self.System.Blok.load_all()
+            self.System.Blok.load_all()
 
         self.loadwithoutmigration = False
 
@@ -996,31 +1006,6 @@ class Registry:
             logger.info('Update session event %r' % funct)
             funct(self.session)
 
-    def run_test(registry, blok2install):
-        startpath = BlokManager.getPath(blok2install)
-
-        class ContextSuite(nose.suite.ContextSuite):
-
-            def __init__(self, *args, **kwargs):
-                super(ContextSuite, self).__init__(*args, **kwargs)
-                if self.context is not None:
-                    self.context.registry = registry
-
-        class ContextSuiteFactory(nose.suite.ContextSuiteFactory):
-            suiteClass = ContextSuite
-
-        class TestLoader(nose.loader.TestLoader):
-
-            def __init__(self, *args, **kwargs):
-                super(TestLoader, self).__init__(*args, **kwargs)
-                self.suiteClass = ContextSuiteFactory(config=self.config)
-
-        for root, dirs, _ in walk(startpath):
-            if 'tests' in dirs:
-                nose.run(defaultTest=[join(root, 'tests')],
-                         testLoader=TestLoader(),
-                         argv=['-v', '-s'])
-
     def assemble_entries(self):
         for entry in RegistryManager.declared_entries:
             if entry in RegistryManager.callback_assemble_entries:
@@ -1044,8 +1029,9 @@ class Registry:
         if self.loadwithoutmigration:
             return
 
-        if not self.withoutautomigration:
-            self.declarativebase.metadata.create_all(self.connection())
+        if not self.withoutautomigration and blok2install == 'anyblok-core':
+            self.declarativebase.metadata.tables['system_blok'].create(
+                bind=self.connection(), checkfirst=True)
 
         self.migration = Configuration.get('Migration', Migration)(self)
         query = """
@@ -1054,21 +1040,40 @@ class Registry:
             WHERE
                 (state = 'toinstall' AND name = '%s')
                 OR state = 'toupdate'""" % blok2install
-        res = self.execute(query).fetchall()
+        res = self.execute(query, fetchall=True)
         if res:
             for blok, installed_version in res:
                 b = BlokManager.get(blok)(self)
-                b.pre_migration(installed_version)
+                parsed_version = (
+                    parse_version(installed_version)
+                    if installed_version is not None
+                    else None)
+                b.pre_migration(parsed_version)
+
+            self.migration.auto_upgrade_database(schema_only=True)
+            if not self.withoutautomigration:
+                self.declarativebase.metadata.create_all(self.connection())
 
             self.migration.auto_upgrade_database()
+
             for blok, installed_version in res:
                 b = BlokManager.get(blok)(self)
-                b.post_migration(installed_version)
+                parsed_version = (
+                    parse_version(installed_version)
+                    if installed_version is not None
+                    else None)
+                b.post_migration(parsed_version)
 
         else:
             self.migration.auto_upgrade_database()
 
-        self.listen_sqlalchemy_known_event()
+    def is_reload_needed(self):
+
+        """Determines whether a reload is needed or not."""
+
+        if self.loadwithoutmigration:
+            return
+
         mustreload = False
         for entry in RegistryManager.declared_entries:
             if entry in RegistryManager.callback_initialize_entries:
@@ -1100,6 +1105,25 @@ class Registry:
 
         self.session.expire(obj, attribute_names=attribute_names)
 
+    def flag_modified(self, obj, attribute_names=None):
+        """ Flag the attributes as modified
+        ::
+
+            registry.flag_modified(instance, ['attr1', 'attr2', ...])
+
+        :param obj: instance of ``Model``
+        :param attribute_names: list of string, names of the attr to expire
+        """
+        if attribute_names:
+            hybrid_property_columns = (
+                obj.__class__.get_hybrid_property_columns()
+            )
+            for attribute_name in attribute_names:
+                if attribute_name in hybrid_property_columns:
+                    attribute_name = anyblok_column_prefix + attribute_name
+
+                flag_modified(obj, attribute_name)
+
     def expire_all(self):
         """Expire all the objects in session
 
@@ -1118,7 +1142,7 @@ class Registry:
         """
         self.session.expunge(obj)
 
-    def refresh(self, obj, attribute_names=None):
+    def refresh(self, obj, attribute_names=None, with_for_update=None):
         """Expire  and reload object in session, you can define some attribute
         which are refreshed::
 
@@ -1126,6 +1150,8 @@ class Registry:
 
         :param obj: instance of ``Model``
         :param attribute_names: list of string, names of the attr to refresh
+        :param with_for_update: Boolean, acquire lock on the row until
+        commit/rollback transaction
         """
         if attribute_names:
             attribute_names = [
@@ -1133,7 +1159,11 @@ class Registry:
                  if x in obj.hybrid_property_columns else x)
                 for x in attribute_names]
 
-        self.session.refresh(obj, attribute_names=attribute_names)
+        self.session.refresh(
+            obj,
+            attribute_names=attribute_names,
+            with_for_update=with_for_update,
+        )
 
     def rollback(self, *args, **kwargs):
         logger.debug('[ROLLBACK] with args=%r and kwargs = %r', args, kwargs)
@@ -1154,7 +1184,7 @@ class Registry:
             session = self.Session()
             session.rollback()
             session.expunge_all()
-            session.close_all()
+            close_all_sessions()
 
         if self.unittest_transaction:
             self.unittest_transaction.close()
@@ -1405,8 +1435,9 @@ class Registry:
                         self.check_conflict_with(blok)
                         self.apply_state(blok, state, ['uninstalled'])
                         upgrade_state_bloks(state)(
-                            self.get_bloks(blok, ['uninstalled'], [
-                                'required', 'optional', 'conditional']))
+                            self.get_bloks(
+                                blok,  ['undefined', 'uninstalled'],
+                                ['required', 'optional', 'conditional']))
                     elif state == 'toupdate':
                         self.apply_state(blok, state, ['installed'])
                         upgrade_state_bloks(state)(
