@@ -15,8 +15,11 @@ from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy import exc as sa_exc, util
 from sqlalchemy_utils.functions import get_class_by_table
 from .field import Field, FieldException
-from .mapper import ModelAdapter, ModelAttribute, ModelRepr
+from .mapper import ModelAdapter, ModelAttribute, ModelRepr, format_schema
 from anyblok.common import anyblok_column_prefix
+from sqlalchemy.ext.orderinglist import OrderingList, _unsugar_count_from
+from types import FunctionType
+
 from logging import getLogger
 
 
@@ -202,7 +205,16 @@ class RelationShip(Field):
 
         :param registry: current registry
         """
-        self.kwargs['collection_class'] = registry.InstrumentedList
+        InstrumentedList = registry.InstrumentedList
+        if self.kwargs.get('collection_class'):
+            collection_class = self.kwargs['collection_class']
+            if isinstance(collection_class, FunctionType):
+                if getattr(
+                    collection_class, 'is_an_anyblok_instrumented_list', False
+                ) is True:
+                    InstrumentedList = self.kwargs['collection_class'](registry)
+
+        self.kwargs['collection_class'] = InstrumentedList
         self.backref_properties['collection_class'] = registry.InstrumentedList
 
     def define_backref_properties(self, registry, namespace, properties):
@@ -229,6 +241,16 @@ class RelationShip(Field):
 
         if isinstance(_backref, (list, tuple)):
             _backref, backref_properties = _backref
+            if backref_properties.get('collection_class'):
+                collection_class = backref_properties['collection_class']
+                if isinstance(collection_class, FunctionType):
+                    if getattr(
+                        collection_class, 'is_an_anyblok_instrumented_list',
+                        False
+                    ) is True:
+                        backref_properties[
+                            'collection_class'] = collection_class(registry)
+
             self.backref_properties.update(backref_properties)
 
         self.define_backref_properties(registry, namespace, properties)
@@ -353,8 +375,11 @@ class Many2One(RelationShip):
         self.kwargs['info']['primary_key'] = self.primary_key
 
         if 'one2many' in kwargs:
-            self.kwargs['backref'] = self.kwargs.pop('one2many')
-            self.kwargs['info']['remote_name'] = self.kwargs['backref']
+            self.kwargs['backref'] = backref = self.kwargs.pop('one2many')
+            self.kwargs['info']['remote_name'] = backref
+            if isinstance(backref, (list, tuple)):
+                self.kwargs['info']['remote_name'] = backref[0]
+                self.backref_properties.update(**backref[1])
 
         self._column_names = None
         if 'column_names' in kwargs:
@@ -407,7 +432,7 @@ class Many2One(RelationShip):
     def update_local_and_remote_columns_names(self, registry, namespace,
                                               fieldname):
         self.remote_columns = self.get_remote_columns(registry)
-        self.kwargs['info']['remote_columns'] = [str(x)
+        self.kwargs['info']['remote_columns'] = [x.attribute_name
                                                  for x in self.remote_columns]
         self.column_names = self.get_columns_names(
             registry, namespace, fieldname, self.remote_columns)
@@ -457,8 +482,8 @@ class Many2One(RelationShip):
                                  "the same, please choose another "
                                  "column_names" % fieldname)
 
-        self.kwargs['info']['local_columns'] = ', '.join(
-            str(x) for x in self.column_names)
+        self.kwargs['info']['local_columns'] = [
+            x.attribute_name for x in self.column_names]
         remote_types = {x.attribute_name: x.native_type(registry)
                         for x in self.remote_columns}
         remote_columns = {x.attribute_name: x
@@ -498,8 +523,17 @@ class Many2One(RelationShip):
             self.fk_names = fk_names
             properties['add_in_table_args'].append(self)
 
-    def update_table_args(self, Model):
+    def remote_model_is_a_table(self, registry):
+        if self.model.model_name not in registry.loaded_namespaces:
+            return True  # by default
+
+        return hasattr(registry.get(self.model.model_name), '__table__')
+
+    def update_table_args(self, registry, Model):
         """Add foreign key constraint in table args"""
+        if not self.remote_model_is_a_table(registry):
+            return []
+
         return [
             ForeignKeyConstraint(self.col_names, self.fk_names,
                                  **self.foreign_key_options)
@@ -529,10 +563,25 @@ class Many2One(RelationShip):
         """
         properties = {
             'fieldname': fieldname, 'relationship_fied': self}
-        InstrumentedList = type(
-            'InstrumentedList', (RelationShipListMany2One, RelationShipList,
-                                 registry.InstrumentedList), properties)
+
+        collection_class = self.backref_properties.get('collection_class', None)
+        if (
+            collection_class
+            and isinstance(collection_class, FunctionType)
+            and getattr(
+                collection_class, 'is_an_anyblok_instrumented_list', False
+            ) is True
+        ):
+            InstrumentedList = collection_class(
+                registry, RelationShipListMany2One, RelationShipList,
+                **properties)
+        else:
+            InstrumentedList = type(
+                'InstrumentedList', (RelationShipListMany2One, RelationShipList,
+                                     registry.InstrumentedList), properties)
+
         self.backref_properties['collection_class'] = InstrumentedList
+
         cascade = self.cascade
         if self.foreign_key_options.get('ondelete') == 'cascade':
             cascade += ', delete'
@@ -571,6 +620,12 @@ class Many2One(RelationShip):
         """
         self.kwargs['foreign_keys'] = '[%s]' % ', '.join(
             [x.get_complete_name(registry) for x in self.column_names])
+        if not self.remote_model_is_a_table(registry):
+            self.kwargs['viewonly'] = True
+            # we used primaryjoin and let the userto defined the good one
+            # The foreign key does not exist, we should fine a good way to
+            # the the local and remote columns
+
         return super(Many2One, self).get_sqlalchemy_mapping(
             registry, namespace, fieldname, properties)
 
@@ -704,7 +759,7 @@ class Many2Many(RelationShip):
                                      many2many="themodels")
 
     if the join_table is not defined, then the table join is
-        "join\_'local table'_and\_'remote table'"
+        "join_'local table'_and_'remote table'"
 
     .. warning::
 
@@ -756,8 +811,13 @@ class Many2Many(RelationShip):
             self.m2m_local_columns = [self.m2m_local_columns]
 
         self.compute_join = self.kwargs.pop('compute_join', False)
-        self.kwargs['backref'] = self.kwargs.pop('many2many', None)
-        self.kwargs['info']['remote_name'] = self.kwargs['backref']
+        self.kwargs['backref'] = backref = self.kwargs.pop('many2many', None)
+        self.kwargs['info']['remote_name'] = backref
+        if isinstance(backref, (list, tuple)):
+            self.kwargs['info']['remote_name'] = backref[0]
+            self.backref_properties.update(**backref[1])
+
+        self.schema = self.kwargs.pop('schema', None)
 
     def autodoc_get_properties(self):
         res = super(Many2Many, self).autodoc_get_properties()
@@ -767,6 +827,9 @@ class Many2Many(RelationShip):
         if self.join_model:
             res['join model'] = self.join_model.model_name
 
+        if self.schema:
+            res['schema'] = self.schema
+
         res['remote_columns'] = self.remote_columns
         res['m2m_remote_columns'] = self.m2m_remote_columns
         res['local_columns'] = self.local_columns
@@ -778,7 +841,8 @@ class Many2Many(RelationShip):
                         suffix=""):
         if m2m_columns is None:
             m2m_columns = [
-                x.get_fk_name(registry).replace('.', '_') + suffix
+                x.get_fk_name(
+                    registry, with_schema=False).replace('.', '_') + suffix
                 for x in columns]
         elif self.join_model:
             m2m_columns_ = []
@@ -863,10 +927,16 @@ class Many2Many(RelationShip):
         if self.join_model:
             join_model_table = self.join_model.tablename(registry)
 
+        if join_table and '.' in join_table:
+            schema, table = join_table.split('.')
+            schema = format_schema(schema, namespace)
+            join_table = '%s.%s' % (schema, table)
+
         if join_table is None and join_model_table is None:
-            join_table = 'join_%s_and_%s' % (
-                self.local_model.tablename(registry),
-                self.model.tablename(registry))
+            join_table = ('join_%s_and_%s_for_%s' % (
+                self.local_model.tablename(registry, with_schema=False),
+                self.model.tablename(registry, with_schema=False),
+                fieldname))[:63]
 
         elif join_table and join_model_table and join_table != join_model_table:
             raise FieldException(
@@ -881,6 +951,27 @@ class Many2Many(RelationShip):
 
         return join_table or join_model_table
 
+    def has_join_table_for_schema(self, registry, namespace, properties,
+                                  join_table):
+        has_join_table = False
+        schema = None
+        tables = registry.declarativebase.metadata.tables
+        if '.' in join_table:
+            has_join_table = join_table in tables
+        elif self.join_model:
+            has_join_table = join_table in tables
+        elif self.schema:
+            schema = format_schema(self.schema, namespace)
+            has_join_table = self.schema + '.' + join_table in tables
+        elif properties.get('__db_schema__'):
+            schema = properties['__db_schema__']
+            has_join_table = (
+                properties['__db_schema__'] + '.' + join_table in tables)
+        elif join_table in tables:
+            has_join_table = True
+
+        return has_join_table, schema
+
     def get_sqlalchemy_mapping(self, registry, namespace, fieldname,
                                properties):
         """ Create the relationship
@@ -896,7 +987,9 @@ class Many2Many(RelationShip):
         local_columns, remote_columns = self.get_local_and_remote_columns(
             registry)
         join_table = self.get_join_table(registry, namespace, fieldname)
-        if join_table not in registry.declarativebase.metadata.tables:
+        has_join_table, schema = self.has_join_table_for_schema(
+            registry, namespace, properties, join_table)
+        if not has_join_table:
             modelname = ''.join(x.capitalize() for x in join_table.split('_'))
             remote_columns, remote_fk, secondaryjoin = self.get_m2m_columns(
                 registry, remote_columns, self.m2m_remote_columns, modelname,
@@ -908,7 +1001,8 @@ class Many2Many(RelationShip):
             )
 
             Node = Table(join_table, registry.declarativebase.metadata, *(
-                local_columns + remote_columns + [local_fk, remote_fk]))
+                local_columns + remote_columns + [local_fk, remote_fk]),
+                schema=schema)
 
             if namespace == self.model.model_name:
                 type(modelname, (registry.declarativebase,), {
@@ -918,7 +1012,8 @@ class Many2Many(RelationShip):
                 self.kwargs['secondaryjoin'] = secondaryjoin
 
         elif namespace == self.model.model_name or self.compute_join:
-            table = registry.declarativebase.metadata.tables[join_table]
+            table = registry.declarativebase.metadata.tables[
+                '%s.%s' % (schema, join_table) if schema else join_table]
             cls = get_class_by_table(registry.declarativebase, table)
             modelname = ModelRepr(cls.__registry_name__).modelname(registry)
             if (
@@ -951,7 +1046,8 @@ class Many2Many(RelationShip):
             self.kwargs['primaryjoin'] = primaryjoin
             self.kwargs['secondaryjoin'] = secondaryjoin
 
-        self.kwargs['secondary'] = join_table
+        self.kwargs['secondary'] = (
+            '%s.%s' % (schema, join_table) if schema else join_table)
         # definition of expiration
         if self.kwargs.get('backref'):
             self.init_expire_attributes(registry, namespace, fieldname)
@@ -1025,36 +1121,13 @@ class One2Many(RelationShip):
 
         if 'many2one' in kwargs:
             self.kwargs['backref'] = self.kwargs.pop('many2one')
-            self.kwargs['info']['remote_names'] = self.kwargs['backref']
+            self.kwargs['info']['remote_name'] = self.kwargs['backref']
 
     def autodoc_get_properties(self):
         res = super(One2Many, self).autodoc_get_properties()
         res['remote_columns'] = self.remote_columns
         return res
 
-    def find_foreign_key(self, registry, properties, tablename):
-        """ Return the primary key come from the first step property
-
-        :param registry: the registry which load the relationship
-        :param properties: first step properties for the model
-        :param tablename: the name of the table for the foreign key
-        :rtype: column name of the primary key
-        """
-        fks = []
-        for f, p in properties.items():
-            if f == '__tablename__':
-                continue
-
-            if not hasattr(p, 'foreign_key'):
-                continue
-
-            if p.foreign_key:
-                model = p.foreign_key.model_name
-                if self.get_tablename(registry, model=model) == tablename:
-                    fks.append(f)
-
-        return fks
-
     def add_expire_attributes(self, registry, namespace, fieldname):
         if self.kwargs.get('backref'):
             backref = self.kwargs['backref']
@@ -1077,6 +1150,7 @@ class One2Many(RelationShip):
         self.link_between_columns = [
             (x.attribute_name, x.get_fk_mapper(registry).attribute_name)
             for x in self.remote_columns
+            if x.get_fk_mapper(registry)
         ]
 
         if 'primaryjoin' not in self.kwargs:
@@ -1109,6 +1183,7 @@ class One2Many(RelationShip):
         pjs_ = {}
         self.link_between_columns = []
         self.kwargs['info']['remote_columns'] = []
+        self.kwargs['info']['local_columns'] = []
         for m2o_name, many2one in many2ones:
             remote_columns = many2one.get_remote_columns(registry)
             for x in remote_columns:
@@ -1156,6 +1231,13 @@ class One2Many(RelationShip):
         else:
             self.format_join_and_remote_columns(registry, namespace, fieldname)
 
+        self.kwargs['info']['local_columns'] = []
+        for rcol in self.kwargs['info']['remote_columns']:
+            col = ModelAttribute(
+                self.model.model_name, rcol).get_fk_column(registry)
+            if col:
+                self.kwargs['info']['local_columns'].append(col)
+
         self.add_expire_attributes(registry, namespace, fieldname)
         return super(One2Many, self).get_sqlalchemy_mapping(
             registry, namespace, fieldname, properties)
@@ -1177,3 +1259,26 @@ class One2Many(RelationShip):
     def get_relationship_cls(self):
         self.kwargs['relationship_field'] = self
         return RelationshipProperty
+
+
+def ordering_list(*args, **kwargs):
+    fnct_args = args
+    fnct_kwargs = kwargs
+
+    def wrap(registry, *instrumented_list_bases, **properties):
+        InstrumentedList = type(
+            'InstrumentedList',
+            (
+                OrderingList.__mro__[0],
+                *instrumented_list_bases,
+                registry.InstrumentedList,
+            ),
+            properties
+        )
+
+        kw = _unsugar_count_from(**fnct_kwargs)
+        return lambda: InstrumentedList(*fnct_args, **kw)
+
+    wrap.is_an_anyblok_instrumented_list = True
+
+    return wrap
