@@ -2,6 +2,7 @@
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2019 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
@@ -12,18 +13,21 @@ from anyblok.blok import BlokManager
 from anyblok.config import Configuration, get_db_name
 from anyblok.registry import RegistryManager, return_list
 from anyblok._graphviz import ModelSchema, SQLSchema
-from nose import main
-import sys
-from os.path import join
-from os import walk
 from argparse import RawDescriptionHelpFormatter
 from textwrap import dedent
 from sqlalchemy_utils.functions import create_database
+import warnings
+import sys
+from os.path import join
+from logging import getLogger
+from os import walk
 from anyblok import (
     load_init_function_from_entry_points,
     configuration_post_load,
 )
 
+logger = getLogger(__name__)
+
 Configuration.add_application_properties(
     'createdb',
     [
@@ -51,15 +55,9 @@ Configuration.add_application_properties(
     description="Update a database: install, upgrade or uninstall the bloks "
 )
 
-Configuration.add_application_properties(
-    'nose', ['logging', 'unittest'],
-    prog='AnyBlok nose, version %r' % version,
-    description="Run fonctionnal nosetest of the installed bloks"
-)
-
 Configuration.add_application_properties(
     'interpreter', ['logging', 'interpreter'],
-    prog='AnyBlok interpretor, version %r' % version,
+    prog='AnyBlok interpreter, version %r' % version,
     description="Run an interpreter on the registry",
     formatter_class=RawDescriptionHelpFormatter,
     epilog=dedent("Example\n"
@@ -67,12 +65,12 @@ Configuration.add_application_properties(
                   "  $ anyblok_interpreter [anyblok arguments] \n"
                   "  $ => registry \n"
                   "  ... <registry> \n\n"
-                  "  The interpretor add in the local the registry of "
-                  "the selected database \n\n"
+                  "  The interpreter gives you a python console with the "
+                  "registry of the selected database \n\n"
                   "Note\n"
                   "----\n"
-                  "  if the ipython is in the python path, then "
-                  "the interpretor will be an ipyton interpretor")
+                  "  If 'ipython' is installed, then the interpreter will be "
+                  "an interactive ipython one.")
 )
 
 Configuration.add_application_properties(
@@ -81,6 +79,13 @@ Configuration.add_application_properties(
 )
 
 
+Configuration.add_application_properties(
+    'nose', ['logging', 'unittest'],
+    prog='AnyBlok nose, version %r' % version,
+    description="Run functional nosetest against installed bloks."
+)
+
+
 def anyblok_createdb():
     """Create a database and install blok from config"""
     load_init_function_from_entry_points()
@@ -96,6 +101,9 @@ def anyblok_createdb():
     if registry is None:
         return
 
+    registry.System.Parameter.set(
+        "with-demo", Configuration.get('with_demo', False))
+
     if Configuration.get('install_all_bloks'):
         bloks = registry.System.Blok.list_by_state('uninstalled')
     else:
@@ -118,7 +126,7 @@ def anyblok_updatedb():
     required_update_bloks = []
     for blok in (Configuration.get('install_or_update_bloks') or []):
         if blok in installed_bloks:
-                required_update_bloks.append(blok)
+            required_update_bloks.append(blok)
         elif blok not in toupdate_bloks:
             required_install_bloks.append(blok)
 
@@ -144,35 +152,6 @@ def anyblok_updatedb():
         registry.close()
 
 
-def anyblok_nose():
-    """Run nose unit test for the registry
-    """
-    registry = anyblok.start('nose', useseparator=True, unittest=True)
-
-    defaultTest = []
-    if registry:
-        installed_bloks = registry.System.Blok.list_by_state("installed")
-        selected_bloks = return_list(
-            Configuration.get('selected_bloks')) or installed_bloks
-
-        unwanted_bloks = return_list(
-            Configuration.get('unwanted_bloks')) or []
-
-        defaultTest = []
-        for blok in installed_bloks:
-            if blok not in selected_bloks or blok in unwanted_bloks:
-                continue
-
-            startpath = BlokManager.getPath(blok)
-            for root, dirs, _ in walk(startpath):
-                if 'tests' in dirs:
-                    defaultTest.append(join(root, 'tests'))
-
-        registry.close()  # free the registry to force create it again
-
-    sys.exit(main(defaultTest=defaultTest))
-
-
 def anyblok_interpreter():
     """Execute a script or open an interpreter
     """
@@ -215,3 +194,45 @@ def anyblok2doc():
             dot = SQLSchema(name_, format=format_)
             doc.toSQL(dot)
             dot.save()
+
+
+def anyblok_nose():
+    """Run nose unit test after giving it the registry
+    """
+    warnings.simplefilter('default')
+    warnings.warn(
+        "This script is deprecated and will be removed soon. "
+        "The Nose test machinery has been removed from the framework in order "
+        "to be replaced with Pytest. "
+        "If you need to run your tests with nose, install the Nose package.",
+        DeprecationWarning, stacklevel=2)
+
+    try:
+        from nose import main
+    except ImportError:
+        logger.error('"Nosetest" is not installed, try: pip install nose')
+
+    registry = anyblok.start('nose', useseparator=True, unittest=True)
+
+    if registry:
+        installed_bloks = registry.System.Blok.list_by_state("installed")
+        selected_bloks = return_list(
+            Configuration.get('selected_bloks')) or installed_bloks
+
+        unwanted_bloks = return_list(
+            Configuration.get('unwanted_bloks')) or []
+        unwanted_bloks.extend(['anyblok-core', 'anyblok-test', 'model_authz'])
+
+        defaultTest = []
+        for blok in installed_bloks:
+            if blok not in selected_bloks or blok in unwanted_bloks:
+                continue
+
+            startpath = BlokManager.getPath(blok)
+            for root, dirs, _ in walk(startpath):
+                if 'tests' in dirs:
+                    defaultTest.append(join(root, 'tests'))
+
+        registry.close()  # free the registry to force create it again
+
+    sys.exit(main(defaultTest=defaultTest))
