@@ -3,20 +3,73 @@
 #
 #    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2019 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
+#    Copyright (C) 2020 Jean-Sebastien SUZANNE <js.suzanne@gmail.com>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
-from anyblok.tests.testcase import DBTestCase
+import pytest
+from anyblok.testing import sgdb_in
 from anyblok import Declarations
+from anyblok.config import Configuration
 from sqlalchemy.exc import IntegrityError
 from anyblok.field import FieldException
 from anyblok.column import (
     Integer, String, BigInteger, Float, Decimal, Boolean, DateTime, Date, Time,
-    Sequence)
-from anyblok.relationship import Many2One
+    Sequence, Email, UUID, URL, Country, Color, PhoneNumber, Selection,
+    TimeStamp)
+from anyblok.relationship import Many2One, ordering_list
 from sqlalchemy import ForeignKeyConstraint
 from sqlalchemy.engine.reflection import Inspector
+from .conftest import init_registry_with_bloks, init_registry, reset_db
+
+
+COLUMNS = [
+    Integer,
+    BigInteger,
+    Float,
+    Decimal,
+    String,
+    Boolean,
+    Date,
+    Time,
+    Email,
+    UUID,
+    Country,
+    Selection,
+]
+
+
+if not sgdb_in(['MySQL', 'MariaDB', 'MsSQL']):
+    COLUMNS.extend([DateTime, TimeStamp])
+
+
+try:
+    import colour  # noqa
+    COLUMNS.append(Color)
+except Exception:
+    pass
+
+try:
+    import furl  # noqa
+    if not sgdb_in(['MySQL', 'MariaDB', 'MsSQL']):
+        COLUMNS.append(URL)
+except Exception:
+    pass
+
+
+try:
+    import phonenumbers  # noqa
+    COLUMNS.append(PhoneNumber)
+except Exception:
+    pass
+
+try:
+    import pycountry  # noqa
+    COLUMNS.append(Country)
+except Exception:
+    pass
 
 
 register = Declarations.register
@@ -24,6 +77,24 @@ Model = Declarations.Model
 Mixin = Declarations.Mixin
 
 
+@pytest.fixture(
+    scope="class",
+    params=[
+        ('prefix', 'suffix'),
+        ('', ''),
+    ]
+)
+def db_schema(request, bloks_loaded):
+    Configuration.set('prefix_db_schema', request.param[0])
+    Configuration.set('suffix_db_schema', request.param[1])
+
+    def rollback():
+        Configuration.set('prefix_db_schema', '')
+        Configuration.set('suffix_db_schema', '')
+
+    request.addfinalizer(rollback)
+
+
 def _complete_many2one(**kwargs):
 
     @register(Model)
@@ -43,6 +114,362 @@ def _complete_many2one(**kwargs):
                            one2many="persons", nullable=False)
 
 
+def _complete_many2one_with_ordered_one2many(**kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address,
+                           remote_columns="id", column_names="id_of_address",
+                           one2many=(
+                               "persons",
+                               dict(
+                                   order_by="ModelPerson.name",
+                                   collection_class=ordering_list('name'),
+                               ),
+                           ),
+                           nullable=False)
+
+
+def _complete_many2one_with_schema(**kwargs):
+
+    @register(Model)
+    class Address:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address,
+                           remote_columns="id", column_names="id_of_address",
+                           one2many="persons", nullable=False)
+
+
+def _complete_many2one_with_different_schema1(**kwargs):
+
+    @register(Model)
+    class Address:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+        __db_schema__ = 'test_db_other_m2o_schema'
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address,
+                           remote_columns="id", column_names="id_of_address",
+                           one2many="persons", nullable=False)
+
+
+def _complete_many2one_with_different_schema2(**kwargs):
+
+    @register(Model)
+    class Address:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address,
+                           remote_columns="id", column_names="id_of_address",
+                           one2many="persons", nullable=False)
+
+
+def _minimum_many2one(**kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address)
+
+
+def _many2one_to_polymorphic_models_1(**kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+        type_entity = String(default='shipping')
+
+        @classmethod
+        def define_mapper_args(cls):
+            mapper_args = super(Address, cls).define_mapper_args()
+            mapper_args.update({
+                'polymorphic_on': cls.type_entity,
+            })
+            return mapper_args
+
+    @register(Model.Address, tablename=Model.Address)
+    class Shipping(Model.Address):
+
+        @classmethod
+        def define_mapper_args(cls):
+            mapper_args = super(Shipping, cls).define_mapper_args()
+            mapper_args.update({
+                'polymorphic_identity': 'shipping',
+            })
+            return mapper_args
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address, one2many="persons")
+
+
+def _many2one_to_polymorphic_models_2(**kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+        type_entity = String(default='shipping')
+
+        @classmethod
+        def define_mapper_args(cls):
+            mapper_args = super(Address, cls).define_mapper_args()
+            mapper_args.update({
+                'polymorphic_on': cls.type_entity,
+            })
+            return mapper_args
+
+    @register(Model.Address)
+    class Shipping(Model.Address):
+        id = Integer(primary_key=True, foreign_key=Model.Address.use('id'))
+
+        @classmethod
+        def define_mapper_args(cls):
+            mapper_args = super(Shipping, cls).define_mapper_args()
+            mapper_args.update({
+                'polymorphic_identity': 'shipping',
+            })
+            return mapper_args
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address, one2many="persons")
+
+
+def _minimum_many2one_with_schema(**kwargs):
+
+    @register(Model)
+    class Address:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+        __db_schema__ = 'test_db_m2o_schema'
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address)
+
+
+def _many2one_with_str_model(**kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = Integer(primary_key=True)
+        street = String()
+        zip = String()
+        city = String()
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model='Model.Address')
+
+
+def many2one_on_mapping_model_and_column_1(**kwargs):
+
+    @register(Model, tablename="x")
+    class Address:
+
+        id = Integer(primary_key=True, db_column_name="x1")
+        street = String(db_column_name="x2")
+        zip = String(db_column_name="x3")
+        city = String(db_column_name="x4")
+
+    @register(Model, tablename="y")
+    class Person:
+
+        name = String(primary_key=True, db_column_name="y1")
+        address = Many2One(model=Model.Address)
+
+
+def many2one_on_mapping_model_and_column_2(**kwargs):
+
+    @register(Model, tablename="x")
+    class Address:
+
+        id = Integer(primary_key=True, db_column_name="x1")
+        street = String(db_column_name="x2")
+        zip = String(db_column_name="x3")
+        city = String(db_column_name="x4")
+
+    @register(Model, tablename="y")
+    class Person:
+
+        name = String(primary_key=True, db_column_name="y1")
+        address_id = Integer(db_column_name="y2",
+                             foreign_key=Model.Address.use('id'))
+        address = Many2One(model=Model.Address)
+
+
+@pytest.fixture(
+    scope="class",
+    params=[
+        (_complete_many2one, 'id_of_address', True),
+        (_complete_many2one_with_ordered_one2many, 'id_of_address', True),
+        (_complete_many2one_with_schema, 'id_of_address', True),
+        (_complete_many2one_with_different_schema1, 'id_of_address', True),
+        (_complete_many2one_with_different_schema2, 'id_of_address', True),
+        (_minimum_many2one, 'address_id', False),
+        (_many2one_to_polymorphic_models_1, 'address_id', True),
+        (_many2one_to_polymorphic_models_2, 'address_id', True),
+        (_minimum_many2one_with_schema, 'address_id', False),
+        (_many2one_with_str_model, 'address_id', False),
+        (many2one_on_mapping_model_and_column_1, 'address_id', False),
+        (many2one_on_mapping_model_and_column_2, 'address_id', False),
+    ]
+)
+def registry_many2one(request, bloks_loaded, db_schema):
+    reset_db()
+    registry = init_registry_with_bloks(
+        [], request.param[0])
+    request.addfinalizer(registry.close)
+    return registry, request.param[1], request.param[2]
+
+
+@pytest.mark.relationship
+class TestMany2One:
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_many2one):
+        transaction = registry_many2one[0].begin_nested()
+        request.addfinalizer(transaction.rollback)
+        return
+
+    def test_ok(self, request, registry_many2one):
+        registry, column_name, has_one2many = registry_many2one
+        address_exist = hasattr(registry.Person, 'address')
+        assert address_exist
+        column_name_exist = hasattr(registry.Person, column_name)
+        assert column_name_exist
+
+        address = registry.Address.insert(
+            street='14-16 rue soleillet', zip='75020', city='Paris')
+
+        person = registry.Person.insert(
+            name="Jean-sébastien SUZANNE", address=address)
+
+        if has_one2many:
+            assert address.persons == [person]
+            person.delete()  # issue 68
+            address.expire('persons')
+
+    def test_autodoc(self, registry_many2one):
+        registry, _, _ = registry_many2one
+        registry.loaded_namespaces_first_step['Model.Person'][
+            'address'].autodoc_get_properties()
+
+    def test_expire_field(self, registry_many2one):
+        registry, column_name, _ = registry_many2one
+        id_of_address = registry.expire_attributes['Model.Person'][
+            column_name]
+        assert ('address',) in id_of_address
+
+
+def _auto_detect_type(ColumnType=None, **kwargs):
+
+    @register(Model)
+    class Address:
+
+        id = ColumnType(primary_key=True, **kwargs)
+
+    @register(Model)
+    class Person:
+
+        name = String(primary_key=True)
+        address = Many2One(model=Model.Address)
+
+
+@pytest.fixture(scope="class", params=COLUMNS)
+def registry_many2one_auto_detect(request, bloks_loaded):
+    reset_db()
+    registry = init_registry_with_bloks(
+        [], _auto_detect_type, ColumnType=request.param)
+    request.addfinalizer(registry.close)
+    return registry
+
+
+@pytest.mark.relationship
+class TestMany2OneAutoDetectColumn:
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_many2one_auto_detect):
+        transaction = registry_many2one_auto_detect.begin_nested()
+        request.addfinalizer(transaction.rollback)
+        return
+
+    def test_check_autodetect_type(self, registry_many2one_auto_detect):
+        registry = registry_many2one_auto_detect
+        assert(
+            str(registry.Address.id.property.columns[0].type) ==
+            str(registry.Person.address_id.property.columns[0].type)
+        )
+
+
 def _many2one_with_same_name_for_column_names(**kwargs):
 
     @register(Model)
@@ -61,34 +488,6 @@ def _many2one_with_same_name_for_column_names(**kwargs):
                            column_names="address")
 
 
-def _minimum_many2one(**kwargs):
-
-    @register(Model)
-    class Address:
-
-        id = Integer(primary_key=True)
-
-    @register(Model)
-    class Person:
-
-        name = String(primary_key=True)
-        address = Many2One(model=Model.Address)
-
-
-def _many2one_with_str_model(**kwargs):
-
-    @register(Model)
-    class Address:
-
-        id = Integer(primary_key=True)
-
-    @register(Model)
-    class Person:
-
-        name = String(primary_key=True)
-        address = Many2One(model='Model.Address')
-
-
 def _minimum_many2one_without_model(**kwargs):
 
     @register(Model)
@@ -103,20 +502,6 @@ def _minimum_many2one_without_model(**kwargs):
         address = Many2One()
 
 
-def _auto_detect_type(ColumnType=None, **kwargs):
-
-    @register(Model)
-    class Address:
-
-        id = ColumnType(primary_key=True, **kwargs)
-
-    @register(Model)
-    class Person:
-
-        name = String(primary_key=True)
-        address = Many2One(model=Model.Address)
-
-
 def _two_remote_primary_keys(**kwargs):
 
     @register(Model)
@@ -132,44 +517,29 @@ def _two_remote_primary_keys(**kwargs):
         test = Many2One(model=Model.Test)
 
 
-class TestMany2One(DBTestCase):
+@pytest.mark.relationship
+class TestMany2OneOld:
 
-    def test_complete_many2one(self):
-        registry = self.init_registry(_complete_many2one)
-        address_exist = hasattr(registry.Person, 'address')
-        self.assertTrue(address_exist)
-        id_of_address_exist = hasattr(registry.Person, 'id_of_address')
-        self.assertTrue(id_of_address_exist)
+    @pytest.fixture(autouse=True)
+    def close_registry(self, request, bloks_loaded):
 
-        address = registry.Address.insert(
-            street='14-16 rue soleillet', zip='75020', city='Paris')
+        def close():
+            if hasattr(self, 'registry'):
+                self.registry.close()
 
-        person = registry.Person.insert(
-            name="Jean-sébastien SUZANNE", address=address)
+        request.addfinalizer(close)
 
-        self.assertEqual(address.persons, [person])
-
-    def test_complete_many2one_expire_field(self):
-        registry = self.init_registry(_complete_many2one)
-        self.assertIn(
-            ('address',),
-            registry.expire_attributes['Model.Person']['id_of_address'])
+    def init_registry(self, *args, **kwargs):
+        reset_db()
+        self.registry = init_registry(*args, **kwargs)
+        return self.registry
 
     def test_2_many2one(self):
-        with self.assertRaises(FieldException):
+        with pytest.raises(FieldException):
             self.init_registry(_many2one_with_same_name_for_column_names)
 
-    def test_minimum_many2one(self):
-        registry = self.init_registry(_minimum_many2one)
-        address_exist = hasattr(registry.Person, 'address')
-        self.assertTrue(address_exist)
-        address_id_exist = hasattr(registry.Person, 'address_id')
-        self.assertTrue(address_id_exist)
-        address = registry.Address.insert()
-        person = registry.Person.insert(
-            name="Jean-sébastien SUZANNE", address=address)
-        self.assertEqual(person.address, address)
-
+    @pytest.mark.skipif(sgdb_in(['MySQL', 'MariaDB', 'MsSQL']),
+                        reason='ISSUE #89')
     def test_minimum_many2one_on_sequence(self):
 
         def add_in_registry():
@@ -188,62 +558,12 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert()
         test2 = registry.Test2.insert(test=test)
-        self.assertIs(test, test2.test)
-
-    def test_minimum_many2one_expire_field(self):
-        registry = self.init_registry(_minimum_many2one)
-        self.assertEqual(
-            registry.expire_attributes['Model.Person']['address_id'],
-            {('address',)})
-
-    def test_many2one_with_str_model(self):
-        registry = self.init_registry(_many2one_with_str_model)
-        address_exist = hasattr(registry.Person, 'address')
-        self.assertTrue(address_exist)
-        address_id_exist = hasattr(registry.Person, 'address_id')
-        self.assertTrue(address_id_exist)
-        address = registry.Address.insert()
-        person = registry.Person.insert(
-            name="Jean-sébastien SUZANNE", address=address)
-        self.assertIs(person.address, address)
+        assert test is test2.test
 
     def test_minimum_many2one_without_model(self):
-        with self.assertRaises(FieldException):
+        with pytest.raises(FieldException):
             self.init_registry(_minimum_many2one_without_model)
 
-    def check_autodetect_type(self, ColumnType):
-        registry = self.init_registry(_auto_detect_type, ColumnType=ColumnType)
-        self.assertEqual(
-            str(registry.Address.id.property.columns[0].type),
-            str(registry.Person.address_id.property.columns[0].type))
-
-    def test_autodetect_type_integer(self):
-        self.check_autodetect_type(Integer)
-
-    def test_autodetect_type_big_integer(self):
-        self.check_autodetect_type(BigInteger)
-
-    def test_autodetect_type_float(self):
-        self.check_autodetect_type(Float)
-
-    def test_autodetect_type_decimal(self):
-        self.check_autodetect_type(Decimal)
-
-    def test_autodetect_type_string(self):
-        self.check_autodetect_type(String)
-
-    def test_autodetect_type_boolean(self):
-        self.check_autodetect_type(Boolean)
-
-    def test_autodetect_type_datetime(self):
-        self.check_autodetect_type(DateTime)
-
-    def test_autodetect_type_date(self):
-        self.check_autodetect_type(Date)
-
-    def test_autodetect_type_time(self):
-        self.check_autodetect_type(Time)
-
     def test_same_model(self):
         def add_in_registry():
 
@@ -256,8 +576,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         t1 = registry.Test.insert()
         t2 = registry.Test.insert(parent=t1)
-        self.assertEqual(t2.parent, t1)
-        self.assertEqual(t1.children[0], t2)
+        assert t2.parent is t1
+        assert t1.children[0] is t2
 
     def test_same_model_pk_by_inherit(self):
         def add_in_registry():
@@ -275,8 +595,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         t1 = registry.Test.insert()
         t2 = registry.Test.insert(parent=t1)
-        self.assertEqual(t2.parent, t1)
-        self.assertEqual(t1.children[0], t2)
+        assert t2.parent is t1
+        assert t1.children[0] is t2
 
     def test_same_model_pk_by_mixin(self):
         def add_in_registry():
@@ -294,8 +614,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         t1 = registry.Test.insert()
         t2 = registry.Test.insert(parent=t1)
-        self.assertEqual(t2.parent, t1)
-        self.assertEqual(t1.children[0], t2)
+        assert t2.parent is t1
+        assert t1.children[0] is t2
 
     def test_add_unique_constraint(self):
         def add_in_registry():
@@ -316,7 +636,7 @@ class TestMany2One(DBTestCase):
         registry.Person.insert(
             name="Jean-sébastien SUZANNE", address=address)
 
-        with self.assertRaises(IntegrityError):
+        with pytest.raises(IntegrityError):
             registry.Person.insert(name="Other", address=address)
 
     def test_add_index_constraint(self):
@@ -336,7 +656,7 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         inspector = Inspector(registry.session.connection())
         indexes = inspector.get_indexes(registry.Person.__tablename__)
-        self.assertEqual(len(indexes), 1)
+        assert len(indexes) == 1
 
     def test_add_primary_keys_constraint(self):
         def add_in_registry():
@@ -355,7 +675,7 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         inspector = Inspector(registry.session.connection())
         pks = inspector.get_primary_keys(registry.Person.__tablename__)
-        self.assertIn('address_id', pks)
+        assert 'address_id' in pks
 
     def test_complet_with_multi_foreign_key(self):
 
@@ -388,8 +708,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.test_id)
-        self.assertEqual(test.id2, test2.test_id2)
+        assert test.id == test2.test_id
+        assert test.id2 is test2.test_id2
 
     def test_complet_with_multi_foreign_key_without_constraint(self):
 
@@ -416,8 +736,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.test_id)
-        self.assertEqual(test.id2, test2.test_id2)
+        assert test.id == test2.test_id
+        assert test.id2 is test2.test_id2
 
     def test_with_multi_foreign_key_on_existing_column(self):
 
@@ -442,8 +762,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.test_id)
-        self.assertEqual(test.id2, test2.test_id2)
+        assert test.id == test2.test_id
+        assert test.id2 is test2.test_id2
 
     def test_with_multi_foreign_key_on_existing_column2(self):
 
@@ -468,24 +788,21 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.other_test_id)
-        self.assertEqual(test.id2, test2.other_test_id2)
+        assert test.id == test2.other_test_id
+        assert test.id2 is test2.other_test_id2
 
     def test_with_multi_foreign_key_on_unexisting_column(self):
         registry = self.init_registry(_two_remote_primary_keys)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.test_id)
-        self.assertEqual(test.id2, test2.test_id2)
+        assert test.id == test2.test_id
+        assert test.id2 is test2.test_id2
 
     def test_many2one_with_multi_fk_expire_field(self):
         registry = self.init_registry(_two_remote_primary_keys)
-        self.assertIn(
-            ('test',),
-            registry.expire_attributes['Model.Test2']['test_id'])
-        self.assertIn(
-            ('test',),
-            registry.expire_attributes['Model.Test2']['test_id2'])
+        assert('test',) in registry.expire_attributes['Model.Test2']['test_id']
+        assert ('test',) in registry.expire_attributes[
+            'Model.Test2']['test_id2']
 
     def test_with_multi_foreign_key_on_unexisting_named_column(self):
 
@@ -507,8 +824,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert(id2="10")
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test.id, test2.test_id)
-        self.assertEqual(test.id2, test2.test_id2)
+        assert test.id == test2.test_id
+        assert test.id2 is test2.test_id2
 
     def test_with_multi_foreign_key_on_unexisting_named_column2(self):
 
@@ -527,7 +844,7 @@ class TestMany2One(DBTestCase):
                 test = Many2One(model=Model.Test, column_names=(
                     'other_test_id', 'other_test_id2'))
 
-        with self.assertRaises(FieldException):
+        with pytest.raises(FieldException):
             self.init_registry(add_in_registry)
 
     def test_m2o_in_mixin(self):
@@ -558,7 +875,7 @@ class TestMany2One(DBTestCase):
         test = registry.Test.insert()
         test1 = registry.Test1.insert(test=test)
         test2 = registry.Test2.insert(test=test)
-        self.assertEqual(test1.test_id, test2.test_id)
+        assert test1.test_id == test2.test_id
 
     def test_delete_m2o_without_fk_options_on_delete_cascade(self):
 
@@ -578,8 +895,8 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert()
         testM2O = registry.TestM2O.insert(test=test)
-        self.assertIs(testM2O.test, test)
-        with self.assertRaises(IntegrityError):
+        assert testM2O.test is test
+        with pytest.raises(IntegrityError):
             test.delete()
 
     def test_delete_m2o_with_fk_options_on_delete_cascade(self):
@@ -601,9 +918,9 @@ class TestMany2One(DBTestCase):
         registry = self.init_registry(add_in_registry)
         test = registry.Test.insert()
         testM2O = registry.TestM2O.insert(test=test)
-        self.assertIs(testM2O.test, test)
+        assert testM2O.test is test
         test.delete()
-        self.assertFalse(registry.TestM2O.query().count())
+        assert not (registry.TestM2O.query().count())
 
     def test_2_simple_many2one_on_the_same_model(self):
 
@@ -626,6 +943,40 @@ class TestMany2One(DBTestCase):
         address_2 = registry.Address.insert()
         person = registry.Person.insert(
             name='test', address_1=address_1, address_2=address_2)
-        self.assertIs(person.address_1, address_1)
-        self.assertIs(person.address_2, address_2)
-        self.assertIsNot(person.address_1, person.address_2)
+        assert person.address_1 is address_1
+        assert person.address_2 is address_2
+        assert person.address_1 is not person.address_2
+
+    def test_many2one_with_order_list_on_one2many(self):
+
+        def add_in_registry():
+
+            @register(Model)
+            class Address:
+
+                id = Integer(primary_key=True)
+
+            @register(Model)
+            class Person:
+
+                id = Integer(primary_key=True)
+                name = String()
+                address = Many2One(
+                    model=Model.Address,
+                    one2many=(
+                        'persons',
+                        dict(
+                            order_by="ModelPerson.name",
+                            collection_class=ordering_list('name'),
+                        )
+                    )
+                )
+
+        registry = self.init_registry(add_in_registry)
+        address = registry.Address.insert()
+        p3 = registry.Person.insert(name='test 3', address=address)
+        p1 = registry.Person.insert(name='test 1', address=address)
+        p4 = registry.Person.insert(name='test 4', address=address)
+        p2 = registry.Person.insert(name='test 2', address=address)
+        assert address.persons == [p1, p2, p3, p4]
+        assert address.persons.ordering_attr == 'name'
